/*
 * PairwiseAnalyzer.java
 *
 * Created on 2005-mar-21
 */

package analyzer;


import java.awt.Color;
import java.awt.Graphics;
import rainbowie.RequirementList;

/**
 * Defines the analyzer for the Pair-Wise Comparision technique
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class PairwiseAnalyzer extends Analyzer {
	
    /**
     * Constructs a Pair-Wise Comparision analyzer
     * 
     * @param rList		   list of Requirements to be analyzed
     * @param canvasWidth  the width of the drawing area
     * @param canvasHeight the height of the drawing area
     */
    public PairwiseAnalyzer(RequirementList rList, int canvasWidth, int canvasHeight) {
        super(rList, canvasWidth, canvasHeight);
	}
    
    /**
	 * Draws support lines in the GraphWindow.
	 * 
	 * @param g 		   the Graphics that will be drawn
	 * @param canvasWidth  the width of the drawing area
	 * @param canvasHeight the height of the drawing area
	 */
    public void drawSupportLines(Graphics g, int canvasWidth, int canvasHeight) {
        g.setColor(Color.gray);
        /* Upper line */
        g.drawLine(40, canvasHeight-40, 40+(canvasWidth-80)/2, 40);
        g.drawString(((int) (maxValue * 10000.0))/100.0+" %", 50, 50);
        /* Lower line */
        g.drawLine(40, canvasHeight-40, canvasWidth-40, 
                canvasHeight-40-(canvasHeight-80)/2);
        g.drawString(((int) (maxValue * 10000.0))/100.0+" %", 
                canvasWidth - 60, canvasHeight-60);
    }
    
    /**
     * Draws relative support lines in the GraphWindow.
     * 
     * @param g 		   the Graphics that will be drawn
	 * @param canvasWidth  the width of the drawing area
	 * @param canvasHeight the height of the drawing area
     */
    public void drawRelativeSupportLines(Graphics g, int canvasWidth,
            int canvasHeight) {
        g.setColor(Color.gray);
        g.drawString(((int) (maxValue * 10000.0))/100.0+" %", 50, 50);
        g.drawString(((int) (maxValue * 10000.0))/100.0+" %", 
                canvasWidth - 60, canvasHeight-60);
        super.drawRelativeSupportLines(g, canvasWidth, canvasHeight);
    }
}