/*
 * ChoosePrioWindow.java
 *
 * Created on Jan 24, 2005
 */

package gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

import rainbowie.*;
import util.Util;

/**
 * Defines a window where the user selects the prioritization
 * technique to use.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class ChoosePrioWindow extends JFrame {

	private Container contentPane;
	private JPanel west, center, east;
	private JPanel definePanel1, definePanel2;
	private JTextField defineField1, defineField2;
	private JToggleButton pgButton, pwcButton, dollarButton;
	private JButton cancelButton, okButton;
	private JMenuItem helpMenuItem;
	private JCheckBox valueCheck, costCheck, riskCheck, defineCheck1, defineCheck2;

	private ActionHandler ah;
	private Rainbowie p;
	
	/**
	 * Constructs a ChoosePrioWindow.
	 * 
	 * @param p reference to the programs Parseq object
	 */
	public ChoosePrioWindow(Rainbowie p) {
		super("Prioritze requirements");
		this.p = p;
		init();
	}

	/**
	 * Initializes the graphical user interface
	 */
	private void init() {
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setSize(400, 200);
		setIconImage(getToolkit().getImage("images\\rainbow.gif"));
		contentPane = this.getContentPane();
		contentPane.setLayout(new BorderLayout());
		ah = new ActionHandler();

		/* West panel */
		west = new JPanel();
		west.setLayout(new BoxLayout(west, BoxLayout.Y_AXIS));
		west.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
		
		pgButton = new JToggleButton("Planning Game");
		Util.setJComponentSize(pgButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		pgButton.addActionListener(ah);
		pgButton.setToolTipText("Use the Planning Game technique");

		pwcButton = new JToggleButton("Pair-Wise Comp.");
		Util.setJComponentSize(pwcButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		pwcButton.addActionListener(ah);
		pwcButton.setToolTipText("Use the Pair-Wise Comparisons technique");
		
		dollarButton = new JToggleButton("$100 Technique");
		Util.setJComponentSize(dollarButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		dollarButton.addActionListener(ah);
		dollarButton.setToolTipText("Use the $100 Technique");
		
		west.add(Box.createVerticalGlue());
		west.add(pgButton);
		west.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
		west.add(pwcButton);
		west.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
		west.add(dollarButton);
		west.add(Box.createVerticalGlue());
		contentPane.add(west, BorderLayout.WEST);

		/* Center panel */
		center = new JPanel();
		center.setLayout(new BoxLayout(center, BoxLayout.Y_AXIS));
		center.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
		
		valueCheck = new JCheckBox("Value");
		valueCheck.setAlignmentX(Component.LEFT_ALIGNMENT);
		valueCheck.setEnabled(false);
		valueCheck.addActionListener(ah);
		valueCheck.setToolTipText("Use value as one priority criteria");
		
		costCheck = new JCheckBox("Cost");
		costCheck.setAlignmentX(Component.LEFT_ALIGNMENT);
		costCheck.setEnabled(false);
		costCheck.addActionListener(ah);
		costCheck.setToolTipText("Use cost as one priority criteria");
		
		riskCheck = new JCheckBox("Risk");
		riskCheck.setAlignmentX(Component.LEFT_ALIGNMENT);
		riskCheck.setEnabled(false);
		riskCheck.addActionListener(ah);
		riskCheck.setToolTipText("Use risk as one priority criteria");
		
		defineCheck1 = new JCheckBox();
		defineCheck1.setAlignmentX(Component.LEFT_ALIGNMENT);
		defineCheck1.setEnabled(false);
		defineCheck1.addActionListener(ah);
		definePanel1 = new JPanel();
		definePanel1.setLayout(new BoxLayout(definePanel1, BoxLayout.X_AXIS));
		defineField1 = new JTextField(15);
		definePanel1.setMinimumSize(new Dimension(150, 20));
		definePanel1.setMaximumSize(new Dimension(150, 20));
		definePanel1.setPreferredSize(new Dimension(150, 20));
		defineField1.setAlignmentX(Component.LEFT_ALIGNMENT);
		definePanel1.add(defineCheck1);
		definePanel1.add(defineField1);
		definePanel1.setAlignmentX(Component.LEFT_ALIGNMENT);
		defineCheck1.setToolTipText("Use your defined priority criteria");
		defineField1.setToolTipText("Define your own priority criteria to use");
		
		defineCheck2 = new JCheckBox();
		defineCheck2.setAlignmentX(Component.LEFT_ALIGNMENT);
		defineCheck2.setEnabled(false);
		defineCheck2.addActionListener(ah);
		definePanel2 = new JPanel();
		definePanel2.setLayout(new BoxLayout(definePanel2, BoxLayout.X_AXIS));
		defineField2 = new JTextField(15);
		definePanel2.setMinimumSize(new Dimension(150, 20));
		definePanel2.setMaximumSize(new Dimension(150, 20));
		definePanel2.setPreferredSize(new Dimension(150, 20));
		defineField2.setAlignmentX(Component.LEFT_ALIGNMENT);
		definePanel2.add(defineCheck2);
		definePanel2.add(defineField2);
		definePanel2.setAlignmentX(Component.LEFT_ALIGNMENT);
		defineCheck2.setToolTipText("Use your defined priority criteria");
		defineField2.setToolTipText("Define your own priority criteria to use");

		center.add(Box.createVerticalStrut(5));
		center.add(valueCheck);
		center.add(Box.createVerticalStrut(5));
		center.add(costCheck);
		center.add(Box.createVerticalStrut(5));
		center.add(riskCheck);
		center.add(Box.createVerticalStrut(5));
		center.add(definePanel1);
		center.add(Box.createVerticalStrut(5));
		center.add(definePanel2);
		center.add(Box.createVerticalStrut(5));

		contentPane.add(center, BorderLayout.CENTER);

		/* East panel */
		east = new JPanel();
		east.setLayout(new BoxLayout(east, BoxLayout.Y_AXIS));
		east.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
		
		cancelButton = new JButton("Cancel");
		cancelButton.setToolTipText("Return to Main window");
		Util.setJComponentSize(cancelButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		cancelButton.addActionListener(ah);
		
		okButton = new JButton("Ok");
		okButton.setToolTipText("Start prioritizing");
		Util.setJComponentSize(okButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		okButton.setEnabled(false);
		okButton.addActionListener(ah);

		east.add(Box.createVerticalGlue());
		east.add(okButton);
		east.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
		east.add(cancelButton);
		east.add(Box.createVerticalGlue());

		contentPane.add(east, BorderLayout.EAST);
		
		/* Set up the menubar */
		JMenuBar menuBar;
		JMenu helpMenu;
		
		menuBar = new JMenuBar();
		helpMenu = new JMenu("Help");
	
		helpMenuItem = new JMenuItem("How to");
		helpMenuItem.addActionListener(ah);
		helpMenu.add(helpMenuItem);
		
		menuBar.add(helpMenu);
		this.setJMenuBar(menuBar);
		
		pack();
		Util.centerJFrame(this);
		setVisible(true);
	}

	/**
	 * ActionListener to handle user interaction.
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class ActionHandler implements ActionListener {
		
		private int prioMethod = -1;

		public void actionPerformed(ActionEvent e) {
		    
			if (e.getSource() == pgButton || e.getSource() == pwcButton
					|| e.getSource() == dollarButton) {
				boolean checks = false;
				if (e.getSource() == pgButton) {
					if (pgButton.isSelected()) {
						pwcButton.setSelected(false);
						dollarButton.setSelected(false);
						checks = true;
						prioMethod = RequirementList.PLANNING_GAME;
					}
				} 
				else if (e.getSource() == pwcButton) {
					if (pwcButton.isSelected()) {
						pgButton.setSelected(false);
						dollarButton.setSelected(false);
						checks = true;
						prioMethod = RequirementList.PAIR_WISE;
					}

				} 
				else if (e.getSource() == dollarButton) {
					if (dollarButton.isSelected()) {
						pgButton.setSelected(false);
						pwcButton.setSelected(false);
						checks = true;
						prioMethod = RequirementList.DOLLAR;
					}
				}
				valueCheck.setSelected(false);
				costCheck.setSelected(false);
				riskCheck.setSelected(false);
				defineCheck1.setSelected(false);
				defineCheck2.setSelected(false);
				okButton.setEnabled(false);
				valueCheck.setEnabled(checks);
				costCheck.setEnabled(checks);
				riskCheck.setEnabled(checks);
				defineCheck1.setEnabled(checks);
				defineCheck2.setEnabled(checks);
			} 
			else if (e.getSource() == valueCheck
					|| e.getSource() == costCheck || e.getSource() == riskCheck
					|| e.getSource() == defineCheck1 || e.getSource() == defineCheck2) {
				/* Count nbr of selected checkboxes */
				int selectedChecks = 0;
				selectedChecks += (valueCheck.isSelected() ? 1 : 0);
				selectedChecks += (costCheck.isSelected() ? 1 : 0);
				selectedChecks += (riskCheck.isSelected() ? 1 : 0);
				selectedChecks += (defineCheck1.isSelected() ? 1 : 0);
				selectedChecks += (defineCheck2.isSelected() ? 1 : 0);
				if (selectedChecks == 2) {
					valueCheck.setEnabled(valueCheck.isSelected());
					costCheck.setEnabled(costCheck.isSelected());
					riskCheck.setEnabled(riskCheck.isSelected());
					defineCheck1.setEnabled(defineCheck1.isSelected());
					defineCheck2.setEnabled(defineCheck2.isSelected());
					defineField1.setEnabled(defineCheck1.isSelected());
					defineField2.setEnabled(defineCheck2.isSelected());
					okButton.setEnabled(true);
				} 
				else {
					valueCheck.setEnabled(true);
					costCheck.setEnabled(true);
					riskCheck.setEnabled(true);
					defineCheck1.setEnabled(true);
					defineCheck2.setEnabled(true);
					okButton.setEnabled(false);
					defineField1.setEnabled(true);
					defineField2.setEnabled(true);

				}
			} 
			else if (e.getSource() == okButton) {
				if (defineCheck1.isSelected()
						&& defineField1.getText().length() == 0) {
					JOptionPane.showMessageDialog(null,
							"Defined criteria cannot be empty",
							"Missing criteria", JOptionPane.ERROR_MESSAGE);
				} 
				else if (defineCheck2.isSelected()
						&& defineField2.getText().length() == 0) {
					JOptionPane.showMessageDialog(null,
							"Defined criteria cannot be empty",
							"Missing criteria", JOptionPane.ERROR_MESSAGE);
				}
				else {
					/* Check which checkBoxes that have been selected */
					String criteria1 = valueCheck.isSelected() ? "Value" : 
						(costCheck.isSelected() ? "Cost" : 
						    (riskCheck.isSelected() ? "Risk" : defineField1.getText()));
					String criteria2 = defineCheck2.isSelected() ? defineField2.getText() :
					    (defineCheck1.isSelected() ? defineField1.getText() :
						(riskCheck.isSelected() ? "Risk" : "Cost"));
					
					p.setPrioParams(prioMethod, criteria1, criteria2);
					/* Continue with selected prioritization method */
					dispose();
				}
			} 
			else if (e.getSource() == cancelButton) {
				dispose();
			}
			else if (e.getSource() == helpMenuItem) {
			    p.showHelpDialog("choosepriowindow.html");
			}
		}
	}
}