/*
 * MainWindow.java
 *
 * Created on Jan 21, 2005
 */

package gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import rainbowie.Rainbowie;
import rainbowie.RequirementList;
import util.*;

/**
 * Defines the MainWindow, this is the first window presented to the user.
 * In the MainWindow, the user can choose what to do and see how far the
 * current process have come.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class MainWindow extends JFrame {
	/* Header names for the tables */
	private String[] colName1 = {"Req. #", "Requirement", "Rel. #"};
	private String[] colName2 = {"Req. #", "Requirement", "Rel. #", "Prio 1", "Prio 2"};
	private int[] importedTableColumnWidth = {0, 0, 0};
	private int[] prioTableColumnWidth = {0, 0, 0, 0, 0};
	
	private Container contentPane;
	
	private ImportWindow iw;
	private Rainbowie p;
	
	private JButton importButton, reprioButton, rootButton,
	quitButton;
	
	private JTable importedTable, prioTable;
	private JScrollPane importScrollPane, prioScrollPane;
	private JPanel left, middle, right; 
	private JLabel prioLabel;
	
	private JMenuItem quitMenuItem, importMenuItem, reprioMenuItem, 
	rootMenuItem, helpMenuItem;
	
	
	/**
	 * Constructs the MainWindow
	 * 
	 * @param p reference to the programs Parseq object
	 */
	public MainWindow(Rainbowie p) {
	    super("Rainbowie parseq");
		
		/* load application icon */
        setIconImage(getToolkit().getImage("images\\rainbow.gif"));
		
		this.p = p;
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		ActionListener ButtonListener = new ActionHandler();
		MouseHandler mh = new MouseHandler();
		iw = new ImportWindow(p);
		
		contentPane = this.getContentPane();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.X_AXIS));
		
		/* Left panel */ 
		left = new JPanel();
		left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
		left.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 5));
		
		importButton = new JButton("Import");
		Util.setJComponentSize(importButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		importButton.addActionListener(ButtonListener);
		importButton.setToolTipText("Import requirements");

		reprioButton = new JButton("Reprioritize");
		Util.setJComponentSize(reprioButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		reprioButton.setEnabled(false);
		reprioButton.addActionListener(ButtonListener);
		reprioButton.setToolTipText("Reprioritize imported requirements");

		rootButton = new JButton("Root-cause");
		Util.setJComponentSize(rootButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		rootButton.setEnabled(false);
		rootButton.addActionListener(ButtonListener);
		rootButton.setToolTipText("Root-cause analysis and improvements elicitation");

		quitButton = new JButton("Quit");
		Util.setJComponentSize(quitButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		quitButton.addActionListener(ButtonListener);
		quitButton.setToolTipText("Quit Rainbowie");
		
		left.add(Box.createVerticalStrut(30));
		left.add(importButton);
		left.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
		left.add(reprioButton);
		left.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
		left.add(rootButton);
		left.add(Box.createVerticalStrut(40));
		left.add(quitButton);
		left.add(Box.createGlue());	
		
		/* Middle panel */
		middle = new JPanel();
		middle.setLayout(new BoxLayout(middle, BoxLayout.Y_AXIS));
		middle.setBorder(BorderFactory.createEmptyBorder(
				10, 5, 10, 5) );

		JLabel importedLabel = new JLabel("Imported Requirements");
		Util.setFont(importedLabel, true, Util.FONT_MEDIUM);
		middle.add(importedLabel);
		middle.add(Box.createVerticalStrut(10));
		importedLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		String rows[][] = new String[60][3];
		DefaultTableModel importedTableModel = new DefaultTableModel(rows, colName1);
		
		/* Set the table and tooltips */
		importedTable = new JTable(importedTableModel) {
	        public Component prepareRenderer(TableCellRenderer renderer,
	                                         int rowIndex, int vColIndex) {
	            Component c = super.prepareRenderer(renderer, rowIndex, vColIndex);
	            if (c instanceof JComponent) {
	                JComponent jc = (JComponent)c;
	                if(getValueAt(rowIndex, vColIndex) != null && 
	                        ((String) getValueAt(rowIndex, vColIndex)).length()>0) {
	                    jc.setToolTipText((String)getValueAt(rowIndex, vColIndex));
	                }
	                else {
	                    jc.setToolTipText(null);
	                }
	            }
	            return c;
	        }
	    };
	    
		importedTable.setRowSelectionAllowed(false);
		importedTable.setColumnSelectionAllowed(false);
		importedTable.setEnabled(false);
		importedTable.getTableHeader().setReorderingAllowed(false);
		importedTable.getTableHeader().addMouseListener(mh);
		importScrollPane = new JScrollPane(importedTable);
		importScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		middle.add(importScrollPane, BorderLayout.CENTER);
		
		/* Right panel */
		right = new JPanel();
		right.setLayout(new BoxLayout(right, BoxLayout.Y_AXIS));
		right.setBorder(BorderFactory.createEmptyBorder(
				10, 5, 10, 10) );
		
		prioLabel = new JLabel("Reprioritized Requirements");
		Util.setFont(prioLabel, true, Util.FONT_MEDIUM);
		right.add(prioLabel);
		right.add(Box.createVerticalStrut(10));
		prioLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		DefaultTableModel prioTableModel = new DefaultTableModel(rows, colName2);
		
		/* Set the table and tooltips */
		prioTable = new JTable(prioTableModel) {
	        public Component prepareRenderer(TableCellRenderer renderer,
	                                         int rowIndex, int vColIndex) {
	            Component c = super.prepareRenderer(renderer, rowIndex, vColIndex);
	            if (c instanceof JComponent) {
	                JComponent jc = (JComponent)c;
	                if(getValueAt(rowIndex, vColIndex) != null && 
	                        ((String) getValueAt(rowIndex, vColIndex)).length()>0) {
	                    jc.setToolTipText((String)getValueAt(rowIndex, vColIndex));
	                }
	                else {
	                    jc.setToolTipText(null);
	                }
	            }
	            return c;
	        }
	    };
	    
		prioTable.setShowHorizontalLines( true );
		prioTable.setRowSelectionAllowed( false );
		prioTable.setColumnSelectionAllowed( false );
		prioTable.setEnabled(false);
		prioTable.getTableHeader().setReorderingAllowed(false);
		prioTable.getTableHeader().addMouseListener(mh);
		
		prioScrollPane = new JScrollPane(prioTable);
		prioScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		right.add(prioScrollPane);
		
		/* Set up the menubar */
		JMenuBar menuBar;
		JMenu fileMenu, windowMenu, helpMenu; 
		menuBar = new JMenuBar();
		fileMenu = new JMenu("File");
		windowMenu = new JMenu("Window");
		helpMenu = new JMenu("Help");
		
		quitMenuItem = new JMenuItem("Quit");
		quitMenuItem.addActionListener(ButtonListener);		
		importMenuItem = new JMenuItem("Import");
		importMenuItem.addActionListener(ButtonListener);
		reprioMenuItem = new JMenuItem("Reprioritize");
		reprioMenuItem.addActionListener(ButtonListener);
		reprioMenuItem.setEnabled(false);
		rootMenuItem = new JMenuItem("Root-cause Analysis");
		rootMenuItem.addActionListener(ButtonListener);
		rootMenuItem.setEnabled(false);
		helpMenuItem = new JMenuItem("How To");
		helpMenuItem.addActionListener(ButtonListener);
		
		menuBar.add(fileMenu);
		fileMenu.add(quitMenuItem);
		menuBar.add(windowMenu);
		windowMenu.add(importMenuItem);
		windowMenu.add(reprioMenuItem);
		windowMenu.add(rootMenuItem);
		menuBar.add(helpMenu);
		helpMenu.add(helpMenuItem);
		
		this.setJMenuBar(menuBar);
		
		contentPane.add(left);
		contentPane.add(middle);
		contentPane.add(right);
		
		this.pack();
		Util.centerJFrame(this);
		
		importedTable.getColumnModel().getColumn(0).setPreferredWidth(50);
		importedTable.getColumnModel().getColumn(1).setPreferredWidth(middle.getWidth() - 118);
		importedTable.getColumnModel().getColumn(2).setPreferredWidth(50);	
		
		prioTable.getColumnModel().getColumn(0).setPreferredWidth(50);
		prioTable.getColumnModel().getColumn(1).setPreferredWidth(right.getWidth() - 218);
		prioTable.getColumnModel().getColumn(2).setPreferredWidth(50);
		prioTable.getColumnModel().getColumn(3).setPreferredWidth(50);
		prioTable.getColumnModel().getColumn(4).setPreferredWidth(50);
		
		saveColumnWidths();
		
		this.setVisible(true);
		
		/* Sets and maintains the minimum size of MainWindow */
		this.addComponentListener(new java.awt.event.ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				JFrame tmp = (JFrame)e.getSource();
				int width = tmp.getWidth();
				int height = tmp.getHeight();
				if (width<750) {
					width = 750;
				}
				if(height<400) {
					height = 400;
				}
				tmp.setSize(width, height);
				repaint();
			}
		});
	}
	
	/**
	 * Add empty rows at the end of a table if rows > JTable.getRowCount()
	 * 
	 * @param table - Table to be padded with rows
	 * @param rows - Total number of rows after padding
	 * @param cols - number of coloums in added rows
	 */
	private void addPaddingToTable(JTable table, int rows, int cols) {
		while (table.getRowCount() < rows) {
			((DefaultTableModel) table.getModel()).addRow(new Object[cols]);
			
		}
	}
	
	/**
	 * Sets the column width of a table.
	 * 
	 * @param table the JTable to set the width for
	 */
	private void setColumnWidth(JTable table) {
		if(table.equals(importedTable)) {
		    importedTable.getColumnModel().getColumn(0)
		    	.setPreferredWidth(importedTableColumnWidth[0]);
		    importedTable.getColumnModel().getColumn(1)
		    	.setPreferredWidth(importedTableColumnWidth[1]);
		    importedTable.getColumnModel().getColumn(2)
		    	.setPreferredWidth(importedTableColumnWidth[2]);
		}
		else {
		     prioTable.getColumnModel().getColumn(0)
		     	.setPreferredWidth(prioTableColumnWidth[0]);
		     prioTable.getColumnModel().getColumn(1)
		     	.setPreferredWidth(prioTableColumnWidth[1]);
		     prioTable.getColumnModel().getColumn(2)
		     	.setPreferredWidth(prioTableColumnWidth[2]);
		     prioTable.getColumnModel().getColumn(3)
		     	.setPreferredWidth(prioTableColumnWidth[3]);
		     prioTable.getColumnModel().getColumn(4)
		     	.setPreferredWidth(prioTableColumnWidth[4]);
		}
	}
	
	/**
	 * Saves the column widths to preserve the column dimensions when 
	 * sorting the tables.
	 */
	private void saveColumnWidths() {
	    importedTableColumnWidth[0] = 
	        importedTable.getColumnModel().getColumn(0).getWidth();
	    importedTableColumnWidth[1] = 
	        importedTable.getColumnModel().getColumn(1).getWidth();
	    importedTableColumnWidth[2] = 
	        importedTable.getColumnModel().getColumn(2).getWidth();	    

	    prioTableColumnWidth[0] = 
	        prioTable.getColumnModel().getColumn(0).getWidth();
	    prioTableColumnWidth[1] = 
	        prioTable.getColumnModel().getColumn(1).getWidth();
	    prioTableColumnWidth[2] = 
	        prioTable.getColumnModel().getColumn(2).getWidth();
	    prioTableColumnWidth[3] = 
	        prioTable.getColumnModel().getColumn(3).getWidth();
	    prioTableColumnWidth[4] = 
	        prioTable.getColumnModel().getColumn(4).getWidth();    
	}
	
	/**
	 * Sets the size of the window. Used for fixing a resizing problem.
	 */
	private void setWindowSize() {
		int horizontalDiff = left.getWidth()/2 + 20;
		int verticalDiff = 130;
		
		middle.setSize(this.getWidth()/2-horizontalDiff-50, this.getHeight()-verticalDiff);
		right.setSize(this.getWidth()/2-horizontalDiff+50, this.getHeight()-verticalDiff);
		
		Util.setJComponentSize(importScrollPane, 
				this.getWidth()/2-horizontalDiff-50, this.getHeight()-verticalDiff);
		Util.setJComponentSize(prioScrollPane, 
				this.getWidth()/2-horizontalDiff+50, this.getHeight()-verticalDiff);
	}

	/**
	 * Makes sure that the window size and column widths are
	 * adjusted before the super-class call to 
	 * JFrame.paint(Graphics g) is done
	 */
	public void paint(Graphics g) {
		setWindowSize();
		
		this.invalidate();
		this.validate();
		
		super.paint(g);
	}
		
	/**
	 * Assigns the values to the cells in importedTable
	 * 
	 * @param rList the list to fetch the values from
	 */
	public void setImportedTable(RequirementList rList) {
	    saveColumnWidths();
		int k = importedTable.getRowCount();
		for(int i=0;i<k;i++) {
			((DefaultTableModel) importedTable.getModel()).removeRow(0);
		}
		String[] newHeader = {"Req. #", "Requirment", "Rel. #"};
		DefaultTableModel model = new DefaultTableModel(newHeader, 1);
		importedTable.setModel(model);
		setColumnWidth(importedTable);
		for(int i=0;i<rList.getLength();i++) {
			Object[] row = new Object[3];			
				row[0] = rList.getReq(i).getReqNbr();
				row[1] = rList.getReq(i).getReq();
				row[2] = rList.getReq(i).getRelNbr();
			((DefaultTableModel) importedTable.getModel()).insertRow(i,row);
		}
		addPaddingToTable(importedTable, 60, 3);
		reprioButton.setEnabled(true);
		reprioMenuItem.setEnabled(true);
	}
	
	
	/**
	 * Assigns the values to the cells in prioTable
	 * 
	 * @param rList the list to fetch the values from
	 */
	public void setPrioTable(RequirementList rList) {
	    saveColumnWidths();
		int k = prioTable.getRowCount();
		for(int i=0;i<k;i++) {
			((DefaultTableModel) prioTable.getModel()).removeRow(0);
		}
		String[] newHeader = {"Req. #", "Requirment", "Rel. #"
				, rList.getPrioCriteria1(), rList.getPrioCriteria2() };
		DefaultTableModel model = new DefaultTableModel(newHeader, 1);
		prioTable.setModel(model);
		setColumnWidth(prioTable);
		for(int i=0;i<rList.getLength();i++) {
			Object[] row = new Object[5];

			if(rList.getPrioMethod() == RequirementList.PLANNING_GAME) {
			    if(rList.getReq(i).getPriority1()!=-1.0) {
			        row[0] = rList.getReq(i).getReqNbr();
			        row[1] = rList.getReq(i).getReq();
			        row[2] = rList.getReq(i).getRelNbr();
			        double prio1 = rList.getReq(i).getPriority1();
			        double prio2 = rList.getReq(i).getPriority2();
			        switch((int) (prio1*10)%10) {
			        case 0: row[3] = "High"; break;
			        case 1: row[3] = "Medium"; break;
			        case 2: row[3] = "Low"; break;
			        }
			        switch((int) (prio2*10)%10) {
			        case 0: row[4] = "High"; break;
			        case 1: row[4] = "Medium"; break;
			        case 2: row[4] = "Low"; break;
			        }
			    }
			}
			else {
			    if(rList.getReq(i).getReqNbr()!=null && rList.getReq(i).getReqNbr().compareTo("")!=0) {
			        row[0] = rList.getReq(i).getReqNbr();
			        row[1] = rList.getReq(i).getReq();
			        row[2] = rList.getReq(i).getRelNbr();
			        row[3] = rList.getReq(i).getPriority1() + "";
			        row[4] = rList.getReq(i).getPriority2() + "";
			    } 
			}
			((DefaultTableModel) prioTable.getModel()).insertRow(i,row);
		}
		addPaddingToTable(prioTable, 60, 5);
		rootButton.setEnabled(true);
		rootMenuItem.setEnabled(true);
		
		/* Set the header, depending on prioritization method used */
		switch ( rList.getPrioMethod() ) {
		case RequirementList.PLANNING_GAME:
		    prioLabel.setText("Reprioritized Requirements - Planning Game");
		    break;
		case RequirementList.DOLLAR:
		    prioLabel.setText("Reprioritized Requirements - $100 Technique");
		    break;
		case RequirementList.PAIR_WISE:
		    prioLabel.setText("Reprioritized Requirements - Pairwise Comparisons");
			break;
		default:
		    prioLabel.setText("Reprioritized Requirements - Unknown Method");
		}
	}
		
	
	/**
	 *  Listener for buttons and menu items.
	 */
	class ActionHandler implements ActionListener {
	    private ChoosePrioWindow cpw;
	    private int graphWindowNbr = 0;
	    
	    public void actionPerformed(ActionEvent e) {
	        if (e.getSource() == importButton || e.getSource() == importMenuItem) {
	            if (p.getImportedList() != null) {
	                iw.updateReqTable(p.getImportedList());
	            }
	            iw.setVisible(true);
	        } 
	        else if (e.getSource() == reprioButton || e.getSource() == reprioMenuItem) {
	            if(cpw!=null) {
	                cpw.dispose();
	            }	                
	            cpw = new ChoosePrioWindow(p);
	        }
	        else if (e.getSource() == rootButton || e.getSource() == rootMenuItem) {
	            new GraphWindow(p.getPrioList(), p, ++graphWindowNbr);
	        }
	        else if (e.getSource() == helpMenuItem) {
	            p.showHelpDialog("mainwindow.html");
	        }
	        else if (e.getSource() == quitButton || e.getSource() == quitMenuItem) {
	            System.exit(0);
	        }
	    }
	}
	
	/**
	 * MouseListener used to detect on which column header the user clicks,
	 * to sort the table after that column.
	 */
	class MouseHandler implements MouseListener, MouseMotionListener{
	    
	    public void mouseClicked(MouseEvent e) {
	        if(e.getClickCount()==1 && e.getSource()==prioTable.getTableHeader()) {
	            int col = prioTable.columnAtPoint(e.getPoint());
	            saveColumnWidths();
	            if(p.getPrioList()!=null) {
	                setPrioTable(p.getPrioList().getSortedList(col));
	            }
	            setColumnWidth(prioTable);
	        }
	        else if(e.getClickCount()==1 && e.getSource()==importedTable.getTableHeader()) {
	            int col = importedTable.columnAtPoint(e.getPoint());
	            saveColumnWidths();
	            if(p.getImportedList()!=null) {
	                setImportedTable(p.getImportedList().getSortedList(col));
	            }
	            setColumnWidth(importedTable);
	        }
	    }
	    
	    public void mouseDragged(MouseEvent e) {  }
	    public void mouseEntered(MouseEvent e) {	}
	    public void mouseExited(MouseEvent e) {	}
	    public void mouseMoved(MouseEvent e) { }
	    public void mousePressed(MouseEvent e) { }
	    public void mouseReleased(MouseEvent e) {  }
	}
}
