/*
 * DollarWindow.java
 *
 * Created on 2005-feb-23
 */

package gui.prio;


import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import prioritizer.DollarTechnique;
import rainbowie.Rainbowie;
import rainbowie.RequirementList;
import util.Util;

/**
 * Defines a $100 window used during the $100 technique.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class DollarWindow extends JFrame {
    private Container contentPane;
    
    private String criteria1, criteria2;
    private JComboBox criteriaSelecter, dollarSelecter;
    private JLabel dollarsUsedLabel;
    private JButton nextUserButton, doneButton, cancelButton, nextCritButton;
    private JPanel north, center, east;
    private JTable reqTable;
    private JMenuItem doneMenuItem, cancelMenuItem, helpMenuItem;
    private JLabel header;
  
    private boolean prioOneDone;
    private boolean keepOrder; // True if $100 is called after PlanningGame
    
    private ActionHandler ah;
    private MouseHandler mh;
    
    private RequirementList rList;
    
    private Rainbowie p;
    private DollarTechnique dt;
    private PlanningGameWindow pgw;
    
    /**
     * Constructs a $100 window used in normal $100 prioritization
     * 
     * @param rList the RequirementList containing the requirements to be 
     * prioritized
     * @param p     reference to the programs Parseq object
     */
    public DollarWindow(RequirementList rList, Rainbowie p) {
        super("$100 Technique");       
        this.criteria1 = rList.getPrioCriteria1();
        this.criteria2 = rList.getPrioCriteria2();
        this.rList = rList;
        this.p = p;
        setIconImage(getToolkit().getImage("images\\rainbow.gif"));
        keepOrder = false;
        prioOneDone = false;
        
        mh = new MouseHandler();
        dt = new DollarTechnique(rList);
        dt.setCriteria(rList.getPrioCriteria1());
        ah = new ActionHandler();
        
        init();
    }
    
    /**
     * Constructs a $100 window used if assigning values to requirements 
     * after a PlanningGame prioritization
     * 
     * @param rList the RequirementList containing the requirements to be 
     * prioritized
     * @param p     reference to the programs Parseq object
     * @param pgw   reference to the PlanningGameWindow
     */
    public DollarWindow(RequirementList rList, Rainbowie p, PlanningGameWindow pgw) {
        super("Planning Game - $100 Technique combination");
        this.pgw = pgw;
        this.criteria1 = rList.getPrioCriteria1();
        this.criteria2 = rList.getPrioCriteria2();
        this.rList = rList;
        this.p = p;
        
        keepOrder = true;
        prioOneDone = false;
        
        mh = new MouseHandler();
        dt = new DollarTechnique(rList);
        dt.setCriteria(rList.getPrioCriteria1());
        ah = new ActionHandler();
        
        init();
    }
 
    /**
	 * Initializes the graphical user interface
	 */
    private void init() {
        setSize(640,480);
        contentPane = this.getContentPane();
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        contentPane.setLayout(new BorderLayout());
        
        /* North */
        north = new JPanel();
        header = new JLabel();
        if(keepOrder && rList.getPrioCriteria1() == "") {
            header.setText(rList.getPrioCriteria2());
        }
        else {
            header.setText(rList.getPrioCriteria1());
        }
        header.setFont(new Font("Dialog", Font.BOLD, 16));
        north.add(header);
        contentPane.add(north, BorderLayout.NORTH);
        
        /* Center */
        center = new JPanel();
        center.setLayout(new BoxLayout(center, BoxLayout.Y_AXIS));
        center.setAlignmentX(Box.RIGHT_ALIGNMENT);
        
        Object[] columnNames = { "Req. #", "Requirement", "$" };
        Object[][] data = getListData();
        RequirementTableModel reqTableModel = new RequirementTableModel(data,
                columnNames);
        
        /* Sets the reqTable and its tooltips */
        reqTable = new JTable(reqTableModel) {
            public Component prepareRenderer(TableCellRenderer renderer,
                    int rowIndex, int vColIndex) {
                Component c = super.prepareRenderer(renderer, rowIndex, vColIndex);
                if (c instanceof JComponent && vColIndex < 2) {
                    JComponent jc = (JComponent)c;
                    if(getValueAt(rowIndex, vColIndex) != null &&
                            ((String) getValueAt(rowIndex, vColIndex)).length()>0) {
                        jc.setToolTipText((String)getValueAt(rowIndex, vColIndex));
                    }
                    else {
                        jc.setToolTipText(null);
                    }
                }
                return c;
            }
        };
        reqTable.getColumnModel().getColumn(0).setMinWidth(50);
        reqTable.getColumnModel().getColumn(0).setPreferredWidth(100);	
        reqTable.getColumnModel().getColumn(1).setMinWidth(50);
        reqTable.getColumnModel().getColumn(1).setPreferredWidth(400);
        reqTable.getColumnModel().getColumn(2).setMinWidth(75);
        reqTable.getColumnModel().getColumn(2).setPreferredWidth(75);
        reqTable.getColumnModel().getColumn(2).setMaxWidth(75);
        
        reqTable.addMouseListener(mh);
        
        JScrollPane scrollPane = new JScrollPane(reqTable);
        center.add(scrollPane, BorderLayout.CENTER);
        reqTable.setEnabled(true);
        reqTable.getTableHeader().setReorderingAllowed(false);
        reqTable.setRowSelectionAllowed(true);
        reqTable.setRowSelectionInterval(0,0);
        
        JLabel dollarTextLabel = new JLabel("$ Used:");
        dollarsUsedLabel = new JLabel("0");
        dollarsUsedLabel.setBorder(BorderFactory.createLoweredBevelBorder());
        Util.setJComponentSize(dollarsUsedLabel, 75, 20);
        dollarsUsedLabel.setHorizontalAlignment(SwingConstants.CENTER);
        
        JPanel centerSouth = new JPanel();
        centerSouth.setLayout(new BoxLayout(centerSouth, BoxLayout.X_AXIS));
        
        centerSouth.add(Box.createHorizontalGlue());
        centerSouth.add(dollarTextLabel);
        centerSouth.add(Box.createRigidArea(new Dimension(10, 0)));
        centerSouth.add(dollarsUsedLabel);
        centerSouth.add(Box.createRigidArea(new Dimension(18, 0)));
        
        center.setBorder(BorderFactory.createEmptyBorder(0, 20, 10, 0));
        center.add(centerSouth, BorderLayout.SOUTH);
        contentPane.add(center, BorderLayout.CENTER);
        
        /* East */
        east = new JPanel();
        east.setLayout(new BoxLayout(east, BoxLayout.Y_AXIS));
        
        JLabel criteriaLabel = new JLabel("Change Prio. Criteria:");
		criteriaLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		criteriaSelecter = new JComboBox();
		Util.setJComponentSize(criteriaSelecter, Util.BUTTON_WIDTH, 20);
		criteriaSelecter.addItem(criteria1);
		criteriaSelecter.addItem(criteria2);
		criteriaSelecter.setAlignmentX(Component.LEFT_ALIGNMENT);
		criteriaSelecter.addActionListener(ah);
		criteriaSelecter.setEnabled(!keepOrder);
		criteriaSelecter.setToolTipText("Change the criteria");
		
		JLabel dollarsLabel = new JLabel("Change dollar limit:");
		dollarsLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		dollarSelecter = new JComboBox();
		Util.setJComponentSize(dollarSelecter, Util.BUTTON_WIDTH, 20);
		dollarSelecter.addItem("100");
		dollarSelecter.addItem("1000");
		dollarSelecter.setAlignmentX(Component.LEFT_ALIGNMENT);
		dollarSelecter.addActionListener(ah);
		dollarSelecter.setToolTipText("Change the dollar limit");
		
		nextCritButton = new JButton("Next Criteria");
        Util.setJComponentSize(nextCritButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
        nextCritButton.setEnabled(false);
        nextCritButton.addActionListener(ah);
        nextCritButton.setToolTipText("Go to the next criteria");
        
        nextUserButton = new JButton("Next User");
        Util.setJComponentSize(nextUserButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
        nextUserButton.setEnabled(false);
        nextUserButton.addActionListener(ah);
        nextUserButton.setToolTipText("Start the prioritization for the next user");
        
        doneButton = new JButton("Done");
        Util.setJComponentSize(doneButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
        doneButton.setEnabled(false);
        doneButton.addActionListener(ah);
        doneButton.setToolTipText("Confirm the priorities");
        
        cancelButton = new JButton("Cancel");
        Util.setJComponentSize(cancelButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
        cancelButton.setEnabled(true);
        cancelButton.addActionListener(ah);
        cancelButton.setToolTipText("Cancel this prioritization");
        
        east.setBorder(BorderFactory.createEmptyBorder(0, 20, 20, 20));
        contentPane.add(east, BorderLayout.EAST);
        
        east.add(criteriaLabel);
        east.add(Box.createVerticalStrut(10));
        east.add(criteriaSelecter);
        east.add(Box.createVerticalStrut(15));
        east.add(dollarsLabel);
        east.add(Box.createVerticalStrut(10));
        east.add(dollarSelecter);
        east.add(Box.createVerticalGlue());
        east.add(nextCritButton);
        east.add(Box.createVerticalStrut(40));
        east.add(nextUserButton);
        east.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
        east.add(doneButton);
        east.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
        east.add(cancelButton);
        east.add(Box.createVerticalStrut(10));
        
        /* Menu */
        JMenuBar menuBar = new JMenuBar();
        JMenu fileMenu = new JMenu("File");
		JMenu helpMenu = new JMenu("Help");
		
		doneMenuItem = new JMenuItem("Done");
		doneMenuItem.addActionListener(ah);
		cancelMenuItem = new JMenuItem("Cancel");
		cancelMenuItem.addActionListener(ah);
		
		helpMenuItem = new JMenuItem("How To");
		helpMenuItem.addActionListener(ah);
		
		fileMenu.add(doneMenuItem);
		fileMenu.add(cancelMenuItem);
		
		menuBar.add(fileMenu);
		menuBar.add(helpMenu);
		helpMenu.add(helpMenuItem);
		
		this.setJMenuBar(menuBar);
        
		Util.centerJFrame(this);
        setVisible(true);
    }
    
    /**
     * Extracts the requirement numbers and requirement descriptions from 
     * the RequirementList.
     * 
     * @return a matrix with with <ol> <li>[i][0] = reqNbr</li>
     * <li>[i][1] = reqDescription</li></ol>
     */
    private Object[][] getListData() {
        Object[][] data = new Object[60][2];
        for (int i=0;i<rList.getLength();i++) {
            data[i][0] = rList.getReq(i).getReqNbr();
            data[i][1] = rList.getReq(i).getReq();
        }
        return data;
    }
    
    /**
     * Clears the table, setting all cells to "" or null.
     * 
     * @param onlyLastColumn true if only last column should be cleared
     */
    private void clearTable(boolean onlyLastColumn) {
        for (int r=0;r<rList.getLength();r++) {
            if (!onlyLastColumn) {
                reqTable.getModel().setValueAt("", r, 0);
                reqTable.getModel().setValueAt("", r, 1);
            }
            reqTable.getModel().setValueAt(null, r, 2);
        }
        dollarsUsedLabel.setText("0");
    }
    
    /**
     * An ActionListener to take care of user interaction
     */
    class ActionHandler implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if(e.getSource() == criteriaSelecter && !prioOneDone) {
                if (header.getText().compareTo(criteria1) == 0) {
					header.setText(criteria2);
					dt.setCriteria(criteria2);
					criteriaSelecter.setSelectedIndex(1);
				} else {
					header.setText(criteria1);
					dt.setCriteria(criteria1);
					criteriaSelecter.setSelectedIndex(0);
				}
            }
            else if(e.getSource() == dollarSelecter) {
                int newValue = (new Integer(dollarSelecter.getSelectedItem().toString())).intValue();
                if(newValue==100 && (new Integer(dollarsUsedLabel.getText())).intValue()>100) {
                    JOptionPane.showMessageDialog(null, 
                            "$100 limit exceeded!",
                            "Failed", JOptionPane.ERROR_MESSAGE);
                    dollarSelecter.setSelectedIndex(1);
                }
                else {
                    dt.setDollarsAvailible(newValue);
                }
            }
            else if (e.getSource() == doneButton || e.getSource() == doneMenuItem) {
                prioOneDone = false;
                if(!keepOrder) {
                    dt.done();
                    p.setPrioList(rList);
                    clearTable(false);
                    dispose();
                }
                else {
                    boolean valid = true;
                    for(int i=0;i<rList.getLength();i++) {
                        if(reqTable.getValueAt(i,2)==null) {
                            valid = false;
                        }
                    }
                    if(valid) {
                        dt.oneCriteriaDone(header.getText());
                        pgw.assignValueDone();
                        clearTable(false);
                        dispose();
                    }
                    else {
                        JOptionPane.showMessageDialog(null, "One or more values are missing",
                                "Values missing", JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
            else if (e.getSource() == nextCritButton) {
                prioOneDone = true;
                dollarSelecter.setEnabled(false);
                nextCritButton.setEnabled(false);
                dt.criteriaDone();
                clearTable(true);
                if (header.getText().compareTo(criteria1) == 0) {
					header.setText(criteria2);
					dt.setCriteria(criteria2);
					criteriaSelecter.setSelectedIndex(1);
				} else {
					header.setText(criteria1);
					dt.setCriteria(criteria1);
					criteriaSelecter.setSelectedIndex(0);
				}
            }
            else if (e.getSource() == nextUserButton) {
                doneButton.setEnabled(false);
                nextUserButton.setEnabled(false);
                dt.nextUser();
                header.setText(criteria1);
                dt.setCriteria(criteria1);
                criteriaSelecter.setSelectedIndex(0);
                clearTable(true);
                criteriaSelecter.setEnabled(true);
                prioOneDone = false;
            }
            else if (e.getSource() == cancelButton || e.getSource() == cancelMenuItem) {
                prioOneDone = false;
                clearTable(false);
                dispose();
            }
            else if (e.getSource() == helpMenuItem) {
			    p.showHelpDialog("dollarwindow.html");
			}
        }
    }

    /**
     * MouseListener used to detect in which cell the user has clicked.
     */
    class MouseHandler implements MouseListener {
		public void mouseClicked(MouseEvent e) {
		    int col = reqTable.columnAtPoint(e.getPoint());
		    int row = reqTable.rowAtPoint(e.getPoint());
		    reqTable.setColumnSelectionInterval(col, col);
		    reqTable.setRowSelectionInterval(row,row);
		}		
		public void mouseEntered(MouseEvent e){	}
		public void mouseExited(MouseEvent e) { }
		public void mousePressed(MouseEvent e) { }
		public void mouseReleased(MouseEvent e) { }
	}
    
    /**
     * DefaultTableModel used for checking the input into the cells. Makes 
     * sure that only valid characters are entered and that the $100 limit 
     * is not exceeded.
     * <br><br>
     * If incorrect values are entered en error message is displayed.
     */
    class RequirementTableModel extends DefaultTableModel {
		
		public RequirementTableModel(Object[][] data, Object[] columnNames) {
			super(data, columnNames);
		}
		
		public boolean isCellEditable(int row, int column) {
			if (column == 2 && ((String) reqTable.getValueAt(row,0))!=null) {
				return true;
			}
			return false;
		}
		
		/**
		 * Set the value in the specified cell. Also makes sure that the value 
		 * is valid for the specified cell.
		 * 
		 * @param value the value to set
		 * @param row   the row of the cell to set the value in
		 * @param col   the column of the cell to set the value in
		 */
		public void setValueAt(Object value, int row, int col) {
		    if (value != null && value != "") {
		        criteriaSelecter.setEnabled(false);  // disable when first value is set
		        try {
		            int newValue = new Integer(value.toString()).intValue();
		            if(newValue < 0) {
		                JOptionPane.showMessageDialog(null, 
		                        "Value must be a positive integer!",
	                            "Invalid number format", JOptionPane.ERROR_MESSAGE);
		            }
		            else {
		                /* Check if the value is valid */
		                if (!keepOrder || allowInsert(newValue, row)) {
		                    int dol = dt.setValue(newValue, row);
		                    /* Check if $100 limit is exceeded */
		                    if( dol > -1 ) {
		                        dollarsUsedLabel.setText(dol+"");
		                        super.setValueAt(value, row, col);
		                        doneButton.setEnabled(false);
		                        nextCritButton.setEnabled(false);
		                        if (dol == dt.getDollarsAvailible()) { // All $ are used
		                            /* 
		                             * If PlanningGame has been used, we 
		                             * do not allow reordering of the 
		                             * priorities.
		                             */
		                            if (keepOrder) { 
		                                doneButton.setEnabled(true);
		                            }
		                            else if (prioOneDone) {
		                                doneButton.setEnabled(true);
		                                nextUserButton.setEnabled(true);
		                            } 
		                            else {
		                                nextCritButton.setEnabled(true);
		                            }
		                        }
		                    }
		                    else {
		                        JOptionPane.showMessageDialog(null, 
		                                "$"+dt.getDollarsAvailible()+" limit exceeded!",
		                                "Failed", JOptionPane.ERROR_MESSAGE);
		                    }
		                }
		                else {
		                    JOptionPane.showMessageDialog(null, 
		                            "The entered value does not\n" +
		                    		"follow the order from the\n" +
		                    		"Planning Game prioritization",
	                                "Illegal value", JOptionPane.ERROR_MESSAGE);
		                }
		            }
		        }
		        catch (NumberFormatException nfe) {
		            JOptionPane.showMessageDialog(null, 
		                    "Value must be a positive integer!",
		                    "Invalid number format", JOptionPane.ERROR_MESSAGE); 
		        }
		    }
		    else if(value == null) {
		        super.setValueAt(value, row, col);
		    }
		}
		
		/**
		 * Check if it is allowed to set the value in the specified row of
		 * the $ column, according to the ordinal scale given by the
		 * PlanningGame.
		 * <br><br>
		 * Only used after the PlanningGame method has been used.
		 * 
		 * @param value the value to check
		 * @param row   the specified row
		 * @return      true if the value is valid for the cell
		 */
		private boolean allowInsert(int value, int row) {
		    for(int i=0;i<rList.getLength();i++) {
		        Object o = reqTable.getValueAt(i, 2);
		        if(o != null) {
		            int oldValue = new Integer(o.toString()).intValue();
		            if(i<row && oldValue<value) {
		                return false;
		            }
		            else if(i>row && oldValue>value) {
		                return false;
		            }
		        }
		    }
		    return true;
		}
	}
}
