/*
 * PairwiseWindow.java
 *
 * Created on 2005-mar-17
 */

package gui.prio;

import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.*;

import prioritizer.Pairwise;
import rainbowie.Rainbowie;
import rainbowie.Requirement;
import rainbowie.RequirementList;
import util.Util;

/**
 * Defines the window used when using the Pair Wise prioritization method.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class PairwiseWindow extends JFrame {
    private JButton backButton, nextButton, cancelButton, doneButton;
    private ButtonGroup buttonGroup;
    private JMenuItem helpMenuItem, variablesMenuItem, doneMenuItem, cancelMenuItem;
    private JCheckBoxMenuItem valuesMenuItem; 
    private JProgressBar progressBar;
    private JLabel header, nbrCompleted;
    private JRadioButton[] radioButtonArray;
    private JPanel radioButtonsPanel;
    private JComboBox criteriaSelecter;
    /*  Used when "deselecting" all radio buttons */
    private JRadioButton mrSecretRadioButton; // Hidden radio button 
    
    private Container contentPane;
   
    private String currentCriteria;
    private boolean prioOneDone;
    
    private ActionHandler ah;
    private MouseHandler mh;
    
    private Rainbowie p;
    private Pairwise pw;
    private PreferencesWindow prefWindow;
    private JTextArea reqArea1, reqArea2;
    private RequirementList rList;
    
    /**
     * Constructs a Pair Wise prioritization window
     * 
     * @param rList the RequirementList containing the requirements to be 
     * prioritized
     * @param p     reference to the programs Parseq object
     */
    public PairwiseWindow(RequirementList rList, Rainbowie p) {
        super("Pairwise Comparisons");
        this.p = p;
        this.rList = rList;
        
        setIconImage(getToolkit().getImage("images\\rainbow.gif"));
        pw = new Pairwise(rList);
        ah = new ActionHandler();
        mh = new MouseHandler();
        prioOneDone = false;
        radioButtonArray = new JRadioButton[17]; 
        currentCriteria = rList.getPrioCriteria1();
        init();
    }
    
    /**
     * Initializes the graphical user interface
     */
    private void init() {
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.setResizable(false);
        contentPane = this.getContentPane();
        contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.X_AXIS));
        
        int n = rList.getLength(); // The number of requirements in the list
        
        /* Left Panel */
        JPanel leftPanel = new JPanel();
        leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));
        leftPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        
        /* Right Panel */
        JPanel rightPanel = new JPanel();
        rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));
        rightPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        
        JLabel jl = new JLabel("Change Prio. Criteria:");
		jl.setAlignmentX(Component.RIGHT_ALIGNMENT);
		criteriaSelecter = new JComboBox();
		Util.setJComponentSize(criteriaSelecter, Util.BUTTON_WIDTH, 20);
		criteriaSelecter.addItem(rList.getPrioCriteria1());
		criteriaSelecter.addItem(rList.getPrioCriteria2());
		criteriaSelecter.setAlignmentX(Component.RIGHT_ALIGNMENT);
		criteriaSelecter.addActionListener(ah);
		criteriaSelecter.setToolTipText("Change the criteria");
        
        /* Header */
        header = new JLabel(currentCriteria);
        header.setFont(new Font("Dialog", Font.BOLD, 24));
        header.setAlignmentX(Box.CENTER_ALIGNMENT);
        
        /* Reqboxes */
        JPanel reqPanel = new JPanel();
        reqPanel.setLayout(new BoxLayout(reqPanel, BoxLayout.X_AXIS));
        reqArea1 = new JTextArea();
        reqArea1.setEditable(false);
        reqArea1.setLineWrap(true);
        reqArea1.setWrapStyleWord(true);
        JScrollPane reqScroll1 = new JScrollPane(reqArea1);
        Util.setJComponentSize(reqScroll1, 200, 250);
        Util.setFont(reqArea1, false, Util.FONT_XSMALL);
        
        reqArea2 = new JTextArea();
        reqArea2.setEditable(false);
        reqArea2.setLineWrap(true);
        reqArea2.setWrapStyleWord(true);
        JScrollPane reqScroll2 = new JScrollPane(reqArea2);
        Util.setJComponentSize(reqScroll2, 200, 250);
        Util.setFont(reqArea2, false, Util.FONT_XSMALL);
        
        reqPanel.add(reqScroll1);
        reqPanel.add(Box.createHorizontalStrut(50));
        reqPanel.add(reqScroll2);
        
        reqArea1.setText(getReqText((Requirement) pw.getCurrentComparisonRow()[1]));
        reqArea2.setText(getReqText((Requirement) pw.getCurrentComparisonRow()[3]));
        
        /* Comparison area */
        radioButtonsPanel = new JPanel();
        radioButtonsPanel.setLayout(new BoxLayout(radioButtonsPanel, BoxLayout.X_AXIS));
        buttonGroup = new ButtonGroup();
        String[] tooltips = {"The left requirement is EXTREMELY more significant",
                "Intermediate value",
                "The left requirement is VERY MUCH more significant",
                "Intermediate value",
                "The left requirement is ESSENTIALLY more significant",
                "Intermediate value",
                "The left requirement is SLIGHTLY more significant",
                "Intermediate value",
                "The requirements are of EQUAL significance",
                "Intermediate value",
                "The right requirement is SLIGHTLY more significant",
                "Intermediate value",
                "The right requirement is ESSENTIALLY more significant",
                "Intermediate value",
                "The right requirement is VERY MUCH more significant",
                "Intermediate value",
        "The right requirement is EXTREMELY more significant" };
        radioButtonsPanel.add(Box.createHorizontalStrut(-1));
        
        for(int i=0;i<radioButtonArray.length;i++) {
            
            radioButtonArray[i] = new JRadioButton();
            radioButtonArray[i].setToolTipText(tooltips[i]);
            radioButtonArray[i].addActionListener(ah);
            radioButtonArray[i].addMouseListener(mh);
            if(i<9) {
                radioButtonArray[i].setActionCommand((9-i)+"");
            }
            else {
                radioButtonArray[i].setActionCommand((7-i)+"");
            }
            buttonGroup.add(radioButtonArray[i]);
            
            radioButtonsPanel.add(radioButtonArray[i]);
            if(i<radioButtonArray.length-1) {
                radioButtonsPanel.add(Box.createHorizontalGlue());                
            }
            
            
        }
        radioButtonsPanel.add(Box.createHorizontalStrut(-3));
        mrSecretRadioButton = new JRadioButton();
        buttonGroup.add(mrSecretRadioButton);
        
        /* Graphics panel */
        JPanel graphicsPanel = new JPanel();
        graphicsPanel.setLayout(new BoxLayout(graphicsPanel, BoxLayout.X_AXIS));
        Image leftImage = Toolkit.getDefaultToolkit().getImage("images\\left_transp.gif");
        Image rightImage = Toolkit.getDefaultToolkit().getImage("images\\right_transp.gif");
        
        JLabel left = new JLabel();
        left.setIcon(new ImageIcon(leftImage));
        
        JLabel right = new JLabel();
        right.setIcon(new ImageIcon(rightImage));
        
        graphicsPanel.add(left);
        graphicsPanel.add(Box.createHorizontalGlue());
        JLabel equals = new JLabel("=");
        equals.setFont(new Font("Dialog", Font.BOLD, 24));
        graphicsPanel.add(equals);
        graphicsPanel.add(Box.createHorizontalGlue());
        graphicsPanel.add(right);
        
        /* Left buttons panel */
        JPanel upperButtonsPanel = new JPanel();
        upperButtonsPanel.setLayout(new BoxLayout(upperButtonsPanel, BoxLayout.X_AXIS));
        
        backButton = new JButton("Back");
        Util.setJComponentSize(backButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
        backButton.addActionListener(ah);
        backButton.setEnabled(false);
        backButton.setToolTipText("Go back one comparison");
        
        nextButton = new JButton("Next");
        Util.setJComponentSize(nextButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
        nextButton.addActionListener(ah);
        nextButton.setEnabled(false);
        nextButton.setToolTipText("Accept and go to the next comparison");
        
        upperButtonsPanel.add(backButton);
        upperButtonsPanel.add(Box.createHorizontalGlue());
        upperButtonsPanel.add(nextButton);
        
        /* Left progress panel */
        JPanel leftProgressPanel = new JPanel();
        leftProgressPanel.setLayout(new BoxLayout(leftProgressPanel, BoxLayout.X_AXIS));
        progressBar = new JProgressBar(0, (n*(n-1))/2);
        progressBar.setValue(0);
        progressBar.setStringPainted(true);
        progressBar.setBorderPainted(false);
        leftProgressPanel.add(progressBar);
        leftProgressPanel.setBorder(BorderFactory.createLoweredBevelBorder());
        
        /* Right Progress Panel */
        JPanel progressPanel2 = new JPanel();
        progressPanel2.setLayout(new BoxLayout(progressPanel2, BoxLayout.X_AXIS));
        progressPanel2.setAlignmentX(Box.RIGHT_ALIGNMENT);
        
        JPanel progressPanel3 = new JPanel();
        progressPanel3.setLayout(new BoxLayout(progressPanel3, BoxLayout.Y_AXIS));
        progressPanel3.setAlignmentX(Box.RIGHT_ALIGNMENT);
        
        JPanel progressPanel4 = new JPanel();
        progressPanel4.setLayout(new BoxLayout(progressPanel4, BoxLayout.Y_AXIS));
        progressPanel4.setAlignmentX(Box.RIGHT_ALIGNMENT);
        
        /* 
         * Minimally needed, completed and how many comparisons that are needed for
         * a full prioritization
         */
        nbrCompleted = new JLabel("Comparisons: 0");
        JLabel neededComp = new JLabel("Needed comparisons: ");
        JLabel nbrReqs = new JLabel("Requirements: " + n);
        JLabel neededMin = new JLabel("- minimal: ");
        JLabel neededMax = new JLabel("- complete: ");
        JLabel neededCompNbr = new JLabel(" ");
        JLabel neededMinNbr = new JLabel(n>2 ? ""+(n) : ""+1);
        JLabel neededMaxNbr = new JLabel(""+(n*(n-1))/2);
        
        nbrCompleted.setAlignmentX(Box.RIGHT_ALIGNMENT);
        neededComp.setAlignmentX(Box.RIGHT_ALIGNMENT);
        nbrReqs.setAlignmentX(Box.RIGHT_ALIGNMENT);
        neededMin.setAlignmentX(Box.RIGHT_ALIGNMENT);
        neededMax.setAlignmentX(Box.RIGHT_ALIGNMENT);
        neededCompNbr.setAlignmentX(Box.RIGHT_ALIGNMENT);
        neededMinNbr.setAlignmentX(Box.RIGHT_ALIGNMENT);
        neededMaxNbr.setAlignmentX(Box.RIGHT_ALIGNMENT);
        
        progressPanel2.add(progressPanel3);
        progressPanel2.add(progressPanel4);
        
        progressPanel3.add(neededComp);
        progressPanel3.add(neededMin);
        progressPanel3.add(neededMax);
        progressPanel4.add(neededCompNbr);
        progressPanel4.add(neededMinNbr);
        progressPanel4.add(neededMaxNbr);
        
        /* Right buttons panel */
        cancelButton = new JButton("Cancel");
        Util.setJComponentSize(cancelButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
        cancelButton.setAlignmentX(Box.RIGHT_ALIGNMENT);
        cancelButton.setToolTipText("Cancel this prioritization");
        
        doneButton = new JButton("Done");
        Util.setJComponentSize(doneButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
        doneButton.setAlignmentX(Box.RIGHT_ALIGNMENT);
        doneButton.setToolTipText("Confirm this prioritization");
        
        cancelButton.addActionListener(ah);
        doneButton.addActionListener(ah);
        doneButton.setEnabled(false);
        
        leftPanel.add(header);
        leftPanel.add(Box.createVerticalStrut(10));
        leftPanel.add(reqPanel);
        leftPanel.add(Box.createVerticalStrut(10));
        leftPanel.add(radioButtonsPanel);
        leftPanel.add(graphicsPanel);
        leftPanel.add(Box.createVerticalStrut(10));
        leftPanel.add(upperButtonsPanel);
        leftPanel.add(Box.createVerticalStrut(16));
        leftPanel.add(leftProgressPanel);
        leftPanel.add(Box.createVerticalStrut(3));
        
        rightPanel.add(Box.createVerticalStrut(40));
        rightPanel.add(jl);
        rightPanel.add(Box.createVerticalStrut(5));
		rightPanel.add(criteriaSelecter);
        rightPanel.add(Box.createVerticalStrut(40));
        rightPanel.add(nbrReqs);
        rightPanel.add(Box.createVerticalStrut(20));
        rightPanel.add(nbrCompleted);
        rightPanel.add(Box.createVerticalStrut(20));
        rightPanel.add(progressPanel2);
        
        rightPanel.add(Box.createVerticalGlue());
        rightPanel.add(doneButton);
        rightPanel.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
        rightPanel.add(cancelButton);
        
        contentPane.add(leftPanel);
        contentPane.add(rightPanel);
        
        /* Menu */
        JMenuBar menuBar = new JMenuBar();
        JMenu fileMenu = new JMenu("File");
        JMenu optionsMenu = new JMenu("Options");
        JMenu helpMenu = new JMenu("Help");
        
        doneMenuItem = new JMenuItem("Done");
        doneMenuItem.addActionListener(ah);
        cancelMenuItem = new JMenuItem("Cancel");
        cancelMenuItem.addActionListener(ah);
        
        valuesMenuItem = new JCheckBoxMenuItem("Intermediate values");
        variablesMenuItem = new JMenuItem("Set variables...");
        helpMenuItem = new JMenuItem("How To");
        
        valuesMenuItem.addActionListener(ah);
        variablesMenuItem.addActionListener(ah);
        helpMenuItem.addActionListener(ah);
        
        menuBar.add(fileMenu);
        menuBar.add(optionsMenu);
        menuBar.add(helpMenu);
        
        fileMenu.add(doneMenuItem);
        fileMenu.add(cancelMenuItem);
        optionsMenu.add(valuesMenuItem);
        optionsMenu.add(variablesMenuItem);
        helpMenu.add(helpMenuItem);
        toggleIntermediateValues();
        this.setJMenuBar(menuBar);
        pack();      
        Util.centerJFrame(this);
        setVisible(true);
    }
    
    /**
     * Changes between displaying and not displaying the radiobuttons 
     * for the intermediate values together with the others.
     */
    private void toggleIntermediateValues() {
        radioButtonsPanel.removeAll();
        radioButtonsPanel.add(Box.createHorizontalStrut(-1));
        for(int i=0;i<radioButtonArray.length;i++) {
            if(i%2==0) {
                radioButtonsPanel.add(radioButtonArray[i]);
                if(i<radioButtonArray.length-1) {
                    radioButtonsPanel.add(Box.createHorizontalGlue());
                }
            }
            else if(valuesMenuItem.isSelected()) {
                radioButtonsPanel.add(radioButtonArray[i]);
                if(i<radioButtonArray.length-1) {
                    radioButtonsPanel.add(Box.createHorizontalGlue());
                }
            }
            
        }
        radioButtonsPanel.add(Box.createHorizontalStrut(-3));
        radioButtonsPanel.validate();
        repaint();
    }
    
    /**
     * Returns requirement number and requirement description as a String
     * in a format adjusted for the infoArea.
     * 
     * @param r the Requirement to display
     * @return  the formatted String
     */
    private String getReqText(Requirement r) {
        if (r==null) {
            return "";
        }
        String s = "Requirement #: " + "\n" + r.getReqNbr() + 
	    "\n\n" + "Requirement: " + "\n" + r.getReq();
        return s;
    }
   
    /**
     * Enables or disables the radio buttons.
     * 
     * @param enabled true if the radio buttons should be enabled
     */
    private void setRadioButtonsEnabled(boolean enabled) {
        for(int i=0;i<radioButtonArray.length;i++) {
            radioButtonArray[i].setEnabled(enabled);
        }
    }
    
    
    class PreferencesWindow extends JFrame {
        
        private Container contentPane;
        private JButton okButton, cancelButton;
        private JTextField allowedPathsInRandomModeTextField, maxPathsBeforeRandomTextField;
        private int allowedPathsInRandomMode, maxPathsBeforeRandom;
        
        public PreferencesWindow(int allowedPathsInRandomMode, int maxPathsBeforeRandom) {
            super("Set variables");
            this.allowedPathsInRandomMode = allowedPathsInRandomMode;
            this.maxPathsBeforeRandom = maxPathsBeforeRandom;
            init();
        }
        
        private void init() {
            this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            this.setResizable(false);
            
            setIconImage(getToolkit().getImage("images\\rainbow.gif"));
            contentPane = this.getContentPane();
            contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
            
            
            
            /* Upper */
            JPanel upperPanel = new JPanel();
            JLabel header = new JLabel("Set variables");
            Util.setFont(header, true, Util.FONT_SMALL);
            upperPanel.add(header);
            
            /* Middle */
            JPanel middlePanel = new JPanel();
            middlePanel.setLayout(new BoxLayout(middlePanel, BoxLayout.X_AXIS));
            
            JPanel leftPanel = new JPanel();
            leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));
            leftPanel.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 0));
            
            allowedPathsInRandomModeTextField = new JTextField(10);
            allowedPathsInRandomModeTextField.setText(allowedPathsInRandomMode+"");
            maxPathsBeforeRandomTextField = new JTextField(10);
            maxPathsBeforeRandomTextField.setText(maxPathsBeforeRandom+"");
            
            leftPanel.add(maxPathsBeforeRandomTextField);
            leftPanel.add(allowedPathsInRandomModeTextField);
            
            
            JPanel rightPanel = new JPanel();
            rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));
            rightPanel.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
            
            JLabel maxPathsBeforeRandomLabel = new JLabel(": allowed paths before switching to random mode");
            JLabel allowedPathsInRandomModeLabel = new JLabel(": number of paths in random mode");
            
            rightPanel.add(maxPathsBeforeRandomLabel);
            rightPanel.add(allowedPathsInRandomModeLabel);
            
            middlePanel.add(leftPanel);
            middlePanel.add(rightPanel);
            
            /* Lower */
            JPanel lowerPanel = new JPanel();
            lowerPanel.setLayout(new BoxLayout(lowerPanel, BoxLayout.X_AXIS));
            lowerPanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 10, 0));
            
            okButton = new JButton("Ok");
            okButton.addActionListener(ah);
            Util.setJComponentSize(okButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
            
            cancelButton = new JButton("Cancel");
            cancelButton.addActionListener(ah);
            Util.setJComponentSize(cancelButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
            
            lowerPanel.add(Box.createHorizontalGlue());
            lowerPanel.add(okButton);
            lowerPanel.add(Box.createHorizontalStrut(Util.HORIZONTAL_BUTTON_DISTANCE));
            lowerPanel.add(cancelButton);
            lowerPanel.add(Box.createHorizontalGlue());
            
            contentPane.add(upperPanel);
            contentPane.add(middlePanel);
            contentPane.add(lowerPanel);
            
            pack();
            Util.centerJFrame(this);
            setVisible(true);
            
        }
        
        public void dispose() {
            ah.actionPerformed(new ActionEvent(cancelButton, 0, null));
        }
        
    }
    
    
    
    /**
     * ActionListener for the buttons, radio buttons and the menu
     */
    class ActionHandler implements ActionListener {     
        public void actionPerformed(ActionEvent e) {
            if (e.getSource() == criteriaSelecter && !prioOneDone) {
                currentCriteria = criteriaSelecter.getSelectedItem().toString();
				header.setText(currentCriteria);
			} 
            else if(e.getSource()==nextButton && 
                    buttonGroup.getSelection().getActionCommand()!=null) {
                criteriaSelecter.setEnabled(false);
                /* get the value associated with the active radio button */
                double val = 
                    (new Double(buttonGroup.getSelection().getActionCommand()))
                    .doubleValue();
                /* Translate from -x to 1/x */
                if(val<0) {
                    val = 1/(val*-1);
                }
                /* 
                 * Sets the value for the current comparision and checks
                 * if there are more comparisons to be done.
                 */
                if(pw.setValue(val)) {
                    reqArea1.setText(getReqText(
                            (Requirement) pw.getCurrentComparisonRow()[1]));
                    reqArea2.setText(getReqText(
                            (Requirement) pw.getCurrentComparisonRow()[3]));
                    int value = pw.getCurrentValue(); 
                    /*
                     * If the user has gone back through the prioritizations
                     * and goes forward again the correct radio button is
                     * selected, since the value already exist.
                     */
                    for(int i=0;i<radioButtonArray.length;i++) {
                        if(radioButtonArray[i].getActionCommand().compareTo(value+"")==0) {
                            radioButtonArray[i].setSelected(true);
                            if((value*-1)%2==0 && !valuesMenuItem.isSelected()) {
                                valuesMenuItem.setSelected(true);
                                ah.actionPerformed(new ActionEvent(valuesMenuItem, 0, null));
                            }
                            
                        }
                        else if(value==0) {
                            /* Deselect "all" radio buttons */
                            mrSecretRadioButton.setSelected(true); // Hidden
                            nextButton.setEnabled(false);
                            nbrCompleted.setText("Comparisons: "+pw.getCurrentRowIndex());
                            progressBar.setValue(pw.getCurrentRowIndex());
                        }
                    }
                    if (progressBar.getValue() > rList.getLength()-1) {
                        doneButton.setEnabled(true);
                    }
                }
                /* Prioritization done */
                else {
                    setRadioButtonsEnabled(false);
                    nextButton.setEnabled(false);
                    doneButton.setEnabled(true);
                    reqArea1.setText("");
                    reqArea2.setText("");
                    mrSecretRadioButton.setSelected(true);
                    nbrCompleted.setText("Comparisons: "+pw.getCurrentRowIndex());
                    progressBar.setValue(pw.getCurrentRowIndex());
                }
                backButton.setEnabled(true);
                
                /* Calculate geometric means */
                if (pw.getCurrentRowIndex()>rList.getLength()-1) {
                    pw.prioDone(currentCriteria);
                }
            }
            else if(e.getSource()==doneButton || e.getSource()==doneMenuItem) {
                int reply = 0;
                /* If not all comparisons are done, notify the user */
                if (progressBar.getValue() < (rList.getLength()*(rList.getLength()-1))/2) {
                    reply = JOptionPane.showConfirmDialog(null, 
                            "The comparison is incomplete, hence the result may not be\n" +
                            "as accurate as if the comparisons are completed." +
                            "\n\nThe consistency ratio for this prioritization is: " +
                            pw.getCR() +
                            "\n\nDo you want to proceed anyway?", 
                            "Confirm incomplete comparison", JOptionPane.YES_NO_OPTION);
                }
                else {
                    reply = JOptionPane.showConfirmDialog(null, 
                            "The consistency ratio for this prioritization is: " +
                            pw.getCR() +
                            "\n\nDo you want to proceed?", 
                            "Accept consistency ratio", JOptionPane.YES_NO_OPTION);
                }
                if (reply == 0) { // Done with this criteria
                    pw.prioDone(currentCriteria);
                    if(prioOneDone) { // Checks if both criteria are done
                        p.setPrioList(rList);
                        dispose();
                    }
                    else if(currentCriteria.compareTo(rList.getPrioCriteria1())==0) {
                        criteriaSelecter.setSelectedIndex(1);
                    }
                    else {
                        criteriaSelecter.setSelectedIndex(0);
                    }
                    setRadioButtonsEnabled(true);
                    doneButton.setEnabled(false);
                    nextButton.setEnabled(false);
                    backButton.setEnabled(false);
                    mrSecretRadioButton.setSelected(true);
                    prioOneDone = true;
                    header.setText(currentCriteria);
                    pw.reset();
                    reqArea1.setText(getReqText((Requirement) pw.getCurrentComparisonRow()[1]));
                    reqArea2.setText(getReqText((Requirement) pw.getCurrentComparisonRow()[3]));
                    progressBar.setValue(0);
                    nbrCompleted.setText("Comparisons: "+pw.getCurrentRowIndex());
                }
            }
            else if(e.getSource()==backButton) {
                int value = pw.backOneStep();
                reqArea1.setText(getReqText((Requirement) pw.getCurrentComparisonRow()[1]));
                reqArea2.setText(getReqText((Requirement) pw.getCurrentComparisonRow()[3]));
                /*
                 * Select the correct radio button if the user goes back
                 * through old comparisons.
                 */
                for(int i=0;i<radioButtonArray.length;i++) {
                    if(radioButtonArray[i].getActionCommand().compareTo(value+"")==0) {
                        radioButtonArray[i].setSelected(true);
                        if((value*-1)%2==0 && !valuesMenuItem.isSelected()) {
                            valuesMenuItem.setSelected(true);
                            ah.actionPerformed(new ActionEvent(valuesMenuItem, 0, null));
                        }
                    }
                }
                if(pw.getCurrentRowIndex()==0) {
                    backButton.setEnabled(false);
                }
                setRadioButtonsEnabled(true);
                nextButton.setEnabled(true);
            }
            else if(e.getSource()==cancelButton || e.getSource()==cancelMenuItem) {
                dispose();
            }
            else if (e.getSource()==helpMenuItem) {
                p.showHelpDialog("pairwisewindow.html");
            }
            else if (e.getSource() == variablesMenuItem) {
                prefWindow = new PreferencesWindow(pw.getAllowedPaths(), pw.getMaxPathsBeforeRandom());
                setEnabled(false);
            }
            else if (e.getSource() == valuesMenuItem) {
                boolean ok = true;
                for(int i=0;i<radioButtonArray.length;i++) {
                    if(!valuesMenuItem.isSelected() && radioButtonArray[i].isSelected() && i%2!=0) {
                        ok = false;
                        JOptionPane.showMessageDialog(null,
                                "Intermediate values cannot be switched off\n" +
                                "when an intermediate value is selected.\n" +
                                "Please select another value.",
                                "Intermediate value selected", JOptionPane.ERROR_MESSAGE);
                        valuesMenuItem.setSelected(true);
                    }
                }
                if(ok) {
                    toggleIntermediateValues();
                }
                
            }     
            else if(e.getSource() instanceof JRadioButton) {
                if(buttonGroup.getSelection()!=null) {
                    nextButton.setEnabled(true);
                }
            }
            else if (prefWindow!=null && 
                    e.getSource() == prefWindow.okButton) {
                try {
                    int allowed = (new Integer(prefWindow.allowedPathsInRandomModeTextField.getText())).intValue();
                    int maxRandom = (new Integer(prefWindow.maxPathsBeforeRandomTextField.getText())).intValue();
                    pw.setAllowedPaths(allowed);
                    pw.setMaxPathsBeforeRandom(maxRandom);
                    prefWindow.dispose();
                    setEnabled(true);
                }
                catch (NumberFormatException nfe) {
                    JOptionPane.showMessageDialog(null, "The defined values must be in integer form",
                            "Invalid value", JOptionPane.ERROR_MESSAGE);
                }
            }
            else if (prefWindow!=null && 
                    e.getSource() == prefWindow.cancelButton) {
                prefWindow.setVisible(false);
                setEnabled(true);
                toFront();
            }
        }
    }
    
    /**
     * MouseListener used to detect double-clicks on the radio buttons.
     */
    class MouseHandler implements MouseListener {
        /**
         * If a radio button was double-clicked, post an ActionEvent
         * corresponding to nextButton.
         */
		public void mouseClicked(MouseEvent e) {
		    /*
	         * double-clicking a radio button does the same thing as
	         * first selecting one then clicking on nextButton
	         */
			if(e.getClickCount()==2 && e.getModifiers() == MouseEvent.BUTTON1_MASK) {
				ah.actionPerformed(new ActionEvent(nextButton, 0, null));		
			}
		}		
		public void mouseEntered(MouseEvent e){ }
		public void mouseExited(MouseEvent e) { }
		public void mousePressed(MouseEvent e) { }
		public void mouseReleased(MouseEvent e) { }
	}
}
