/*
 * PlanningGameWindow.java
 *
 * Created on 2005-jan-27
 */

package gui.prio;

import java.awt.*;
import java.awt.event.*;
import java.util.LinkedList;

import javax.swing.*;

import prioritizer.PlanningGame;
import rainbowie.Rainbowie;
import rainbowie.Requirement;
import rainbowie.RequirementList;
import util.Util;

/**
 * Defines the window used when using the Planning Game prioritization method.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class PlanningGameWindow extends JFrame {

	private Container contentPane;
	private String criteria1, criteria2, currentCriteria;
	private JComboBox criteriaSelecter;
	private JTextArea infoArea;
	private JScrollPane infoScroll;
	private JScrollPane scrollPane;
	private DrawCanvas desk;
	private JLabel header; 
	private JMenuItem nextCriteriaMenuItem, doneMenuItem, cancelMenuItem, helpMenuItem;
	private JButton nextCritButton, doneButton, cancelButton;
	private JPanel north, east, center;
	
	private PileWindow highPile, mediumPile, lowPile;
	
	private Rainbowie p;
	private PlanningGame pg;
	
	private int pileCanvasHeight;
	private int xSize, ySize, deskWidth, deskHeight;	
	private boolean prioOneDone;

	private ActionHandler ah;
	private WindowHandler wh;
	private MouseHandler mh;
	
	/**
	 * Constructs a Planning Game prioritization window
	 * 
	 * @param rList the RequirementList containing the requirements to be 
     * prioritized
     * @param p     reference to the programs Parseq object
	 */
	public PlanningGameWindow(RequirementList rList, Rainbowie p) {
		super("Planning Game");
		this.p = p;
		this.criteria1 = rList.getPrioCriteria1();
		this.criteria2 = rList.getPrioCriteria2();
		setIconImage(getToolkit().getImage("images\\rainbow.gif"));
		currentCriteria = criteria1;
		prioOneDone = false;
		xSize = 800;
		ySize = 600;
		ah = new ActionHandler();
		pg = new PlanningGame(rList);
		mh = new MouseHandler();
		wh = new WindowHandler();
		this.addWindowListener(wh);
		init();
	}

	/**
	 * Initializes the graphical user interface
	 */
	private void init() {
	    
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		this.setSize(xSize, ySize);
		this.setResizable(true);
		contentPane = this.getContentPane();
		contentPane.setLayout(new BorderLayout());
		
		/* North panel */
		header = new JLabel(currentCriteria);
		Util.setFont(header, true, Util.FONT_LARGE);
		north = new JPanel();
		north.add(header);
		contentPane.add(north, BorderLayout.NORTH);
		
		/* Center panel */
		center = new JPanel();
		center.setLayout(new BoxLayout(center, BoxLayout.X_AXIS));
		
		desk = new DrawCanvas(DrawCanvas.DESK);
		desk.addMouseListener(mh);
		desk.addMouseMotionListener(mh);
		
		scrollPane = new JScrollPane(desk, 
		        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
		        JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		center.add(scrollPane);
		contentPane.add(center, BorderLayout.CENTER);
		
		/* East panel */
		east = new JPanel();
		east.setLayout(new BoxLayout(east, BoxLayout.Y_AXIS));
		east.setBorder(BorderFactory.createEmptyBorder(10, 30, 4, 20) );
		JLabel jl = new JLabel("Change Prio. Criteria:");
		jl.setAlignmentX(Component.LEFT_ALIGNMENT);
		criteriaSelecter = new JComboBox();
		Util.setJComponentSize(criteriaSelecter, Util.BUTTON_WIDTH, 20);
		criteriaSelecter.addItem(criteria1);
		criteriaSelecter.addItem(criteria2);
		criteriaSelecter.setAlignmentX(Component.LEFT_ALIGNMENT);
		criteriaSelecter.addActionListener(ah);
		criteriaSelecter.setToolTipText("Change the criteria");
		
		infoArea = new JTextArea("Requirement #: " + "\n" +
	    	    "\n\n" + "Requirement: " + "\n");
		infoArea.setLineWrap(true);
		infoArea.setWrapStyleWord(true);
		Util.setFont(infoArea, false, Util.FONT_XSMALL);
		infoArea.setEditable(false);
		infoScroll = new JScrollPane(infoArea);
		Util.setJComponentSize(infoScroll, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT*8);
		infoScroll.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		nextCritButton = new JButton("Next Criteria");
		Util.setJComponentSize(nextCritButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		nextCritButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		nextCritButton.setEnabled(false);
		nextCritButton.addActionListener(ah);
		nextCritButton.setToolTipText("Go to the next criteria");
		doneButton = new JButton("Done");
		Util.setJComponentSize(doneButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		doneButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		doneButton.setEnabled(false);
		doneButton.addActionListener(ah);
		doneButton.setToolTipText("Confirm this prioritization");
		cancelButton = new JButton("Cancel");
		Util.setJComponentSize(cancelButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		cancelButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		cancelButton.addActionListener(ah);
		cancelButton.setToolTipText("Cancel this prioritization");
		
		east.add(Box.createVerticalStrut(0));
		east.add(jl);
		east.add(Box.createVerticalStrut(10));
		east.add(criteriaSelecter);
		east.add(Box.createVerticalStrut(40));
		east.add(infoScroll);
		east.add(Box.createVerticalGlue());
		east.add(nextCritButton);
		east.add(Box.createVerticalStrut(40));
		east.add(doneButton);
		east.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
		east.add(cancelButton);
		east.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
		contentPane.add(east, BorderLayout.EAST);
		
		/* Menu */
		JMenuBar menuBar;
		JMenu fileMenu, helpMenu;
		
		menuBar = new JMenuBar();
		fileMenu = new JMenu("File");
		helpMenu = new JMenu("Help");
	
		nextCriteriaMenuItem  = new JMenuItem("Next Criteria");
		nextCriteriaMenuItem.setEnabled(false);
		nextCriteriaMenuItem.addActionListener(ah);
		doneMenuItem = new JMenuItem("Done");
		doneMenuItem.setEnabled(false);
		doneMenuItem.addActionListener(ah);
		cancelMenuItem = new JMenuItem("Cancel");
		cancelMenuItem.addActionListener(ah);
		
		helpMenuItem = new JMenuItem("How To");
		helpMenuItem.addActionListener(ah);
		
		menuBar.add(fileMenu);
		fileMenu.add(nextCriteriaMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(doneMenuItem);
		fileMenu.add(cancelMenuItem);
		menuBar.add(helpMenu);
		helpMenu.add(helpMenuItem);
		
		this.setJMenuBar(menuBar);
		setVisible(true);
		
		this.setExtendedState(MAXIMIZED_BOTH);
		
		/* Let the background be set and drawn before drawing the cards */
		new Thread() {
		    public void run() {
		        try { sleep(500); } catch (InterruptedException ie) { }
		        deskWidth = scrollPane.getWidth();
				deskHeight = scrollPane.getHeight();
				Util.setJComponentSize(desk, deskWidth, deskHeight);
		        pg.makeCards(deskWidth, deskHeight);
		        highPile = new PileWindow("High", pg.getPile("High"));
				mediumPile = new PileWindow("Medium", pg.getPile("Medium"));
				lowPile = new PileWindow("Low", pg.getPile("Low"));
		        desk.repaint();
		    }
		}.start();
		
	}
	/**
	 * Sets requirement number and requirement description in the
	 * info area in an adjusted format.
	 * 
	 * @param r the requirement to display
	 */
	private void setInfoAreaText(Requirement r) { 
	    if(r!=null) {
	        String s = "Requirement #: " + "\n" + r.getReqNbr() +
	        "\n\n" + "Requirement: " + "\n" + r.getReq();
	        infoArea.setText(s);
	        infoArea.setCaretPosition(0);
	    }
	    else {
	        String s = "Requirement #: " + "\n" + 
	        "\n\n" + "Requirement: ";
	        infoArea.setText(s);   
	    }
	}
	
	/**
	 * Method to reach a reference to the PlanningGameWindow
	 * 
	 * @return a reference to the PlanningGameWindow
	 */
	private PlanningGameWindow getWindow() {
	    return this;
	}
	
	/**
	 * Used when values have been assigned with $100 technique
	 * after a Planning Game prioritization
	 */
	public void assignValueDone() {
	    /* The prioMethod of the current RequirementList is changed*/
	    pg.getRequirementList().setPrioMethod(RequirementList.DOLLAR);
	    if (!prioOneDone) {
	        prioOneDone = true;
	        /* Setup for next criteria */
			pg.setCurrentPrioritiesAndShuffle(currentCriteria, true);
			if (currentCriteria.compareTo(criteria1) == 0) {
				currentCriteria = criteria2;
				criteriaSelecter.setSelectedIndex(1);
			} else {
				currentCriteria = criteria1;
				criteriaSelecter.setSelectedIndex(0);
			}
			header.setText(currentCriteria);
			nextCritButton.setEnabled(false);
			nextCriteriaMenuItem.setEnabled(false);
			highPile.repaint();
			mediumPile.repaint();
			lowPile.repaint();
			desk.repaint();
	    }
	    else {
	        /* Set final priorities in Parseq */
	        p.setPrioList(pg.getRequirementList());
	        highPile.dispose();
	        mediumPile.dispose();
	        lowPile.dispose();
	        dispose();
	    }
	    
	}
	
	/**
	 * ActionListener to take care of user interaction
	 */
	class ActionHandler implements ActionListener {
	    
	    private ConfirmWindow cw;
	    
		public void actionPerformed(ActionEvent e) {
		    /*
		     * Buttons and menus in main planning game window
		     */
			if (e.getSource() == criteriaSelecter && !prioOneDone) {
				if (currentCriteria.compareTo(criteria1) == 0) {
					currentCriteria = criteria2;
					criteriaSelecter.setSelectedIndex(1);
				} else {
					currentCriteria = criteria1;
					criteriaSelecter.setSelectedIndex(0);
				}
				header.setText(currentCriteria);
			} 			
			else if (e.getSource() == nextCritButton || 
			        e.getSource() == nextCriteriaMenuItem) {
			    cw = new ConfirmWindow();
				
			} 
			else if (e.getSource() == doneButton || e.getSource() == doneMenuItem) {
			    cw = new ConfirmWindow();				
			} 
			else if (e.getSource() == cancelButton || e.getSource() == cancelMenuItem) {
				highPile.dispose();
				mediumPile.dispose();
				lowPile.dispose();
				dispose();				
			}
			else if (e.getSource() == helpMenuItem) {
			    p.showHelpDialog("planninggamewindow.html");
			}
			
			/*
			 * Buttons in pile windows
			 */
			else if (e.getSource() == highPile.pileOkButton) {
				highPile.setVisible(false);			
			} 
			else if (e.getSource() == mediumPile.pileOkButton) {
				mediumPile.setVisible(false);			
			} 
			else if (e.getSource() == lowPile.pileOkButton) {
				lowPile.setVisible(false);			
			} 
			else if (e.getSource() == highPile.pileToDeskButton) {
				pg.sendToDesk(PlanningGame.HIGH);
				highPile.resizePileCanvas(); 
				highPile.repaint();
				desk.repaint();
				if(pg.getNbrOfCardsOnDesk()!=0) {
					nextCritButton.setEnabled(false);
					nextCriteriaMenuItem.setEnabled(false);
					doneButton.setEnabled(false);
					doneMenuItem.setEnabled(false);
				}
			} 
			else if (e.getSource() == mediumPile.pileToDeskButton) {
				pg.sendToDesk(PlanningGame.MEDIUM);
				mediumPile.resizePileCanvas(); 
				mediumPile.repaint();
				desk.repaint();
				if(pg.getNbrOfCardsOnDesk()!=0) {
					nextCritButton.setEnabled(false);
					nextCriteriaMenuItem.setEnabled(false);
					doneButton.setEnabled(false);
					doneMenuItem.setEnabled(false);
				}
			} 
			else if (e.getSource() == lowPile.pileToDeskButton) {
				pg.sendToDesk(PlanningGame.LOW);
				lowPile.resizePileCanvas(); 
				lowPile.repaint();
				desk.repaint();
				if(pg.getNbrOfCardsOnDesk()!=0) {
					nextCritButton.setEnabled(false);
					nextCriteriaMenuItem.setEnabled(false);
					doneButton.setEnabled(false);
					doneMenuItem.setEnabled(false);
				}
			} 
			
			/*
			 * Buttons in confirm window
			 */
			else if (e.getSource() == cw.assignValuesButton) {
			    RequirementList ordinalList = new RequirementList();
			    ordinalList.setPrioMethod(RequirementList.DOLLAR);
			    if(currentCriteria.compareTo(criteria1) == 0) {
			        ordinalList.setPrioCriteria1(criteria1);
			        ordinalList.setPrioCriteria2("");
			    }
			    else {
			        ordinalList.setPrioCriteria2(criteria2);
			        ordinalList.setPrioCriteria1("");
			    }
			    for(int h=0;h<pg.getPile("High").size();h++) {
		            ordinalList.addRequirement(pg.getRequirement("High", h));
		        }
		        for(int m=0;m<pg.getPile("Medium").size();m++) {
		            ordinalList.addRequirement(pg.getRequirement("Medium", m));
		        }
		        for(int l=0;l<pg.getPile("Low").size();l++) {
		            ordinalList.addRequirement(pg.getRequirement("Low", l));
		        }
		        new DollarWindow(ordinalList, p, getWindow());
		        cw.dispose();
			} 
			else if(e.getSource() == cw.acceptPrioButton) {
			    if (!prioOneDone) {
			        prioOneDone = true;
					pg.setCurrentPrioritiesAndShuffle(currentCriteria, false);
					if (currentCriteria.compareTo(criteria1) == 0) {
						currentCriteria = criteria2;
						criteriaSelecter.setSelectedIndex(1);
					} else {
						currentCriteria = criteria1;
						criteriaSelecter.setSelectedIndex(0);
					}
					header.setText(currentCriteria);
					nextCritButton.setEnabled(false);
					nextCriteriaMenuItem.setEnabled(false);
					highPile.repaint();
					mediumPile.repaint();
					lowPile.repaint();
					
					desk.repaint();
			    }
			    else {
			        pg.setCurrentPrioritiesAndShuffle(currentCriteria, false);
			        p.setPrioList(pg.getRequirementList());
			        
			        highPile.dispose();
			        mediumPile.dispose();
			        lowPile.dispose();
			        dispose();
			    }
			    cw.dispose();
			} 
			else if(e.getSource() == cw.backPrioButton) {
			    cw.dispose();
			}
		}
	}
	
	
	/**
	 * Defines the canvas used for drawing the desk with the cards and 
	 * every pile of cards
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class DrawCanvas extends JPanel {
		private static final int DESK = 0, PILE = 1;
		private Image background;

		private Point startSelectionPoint = null;
		private Point endSelectionPoint = null;
		
		private int pile; // High, Medium or Low pile.
		private int type; // DESK or PILE type of canvas
		
		/**
		 * Constructs a canvas for the desk
		 * 
		 * @param type the current type - 0 for desk
		 */
		public DrawCanvas(int type) {
			super();
			pile = PlanningGame.DESK;
			this.type = type;
			background = Toolkit.getDefaultToolkit().getImage("images\\desk.gif");
		    MediaTracker mediaTracker = new MediaTracker(this);
			mediaTracker.addImage(background, 0);
			try	{
				mediaTracker.waitForID(0);
				if(mediaTracker.isErrorID(0)) {
				    throw new InterruptedException("Unable to set background image");
				}
			} 
			catch (InterruptedException ie) {
				JOptionPane.showMessageDialog(null, "Unable to set background image",
						"Failed", JOptionPane.ERROR_MESSAGE);
				ie.printStackTrace();
			}
		}
		
		/**
		 * Constructs a canvas for a pile
		 *  
		 * @param type the current type - 1 for pile
		 * @param pile used to tell if the current pile is 
		 * the High, Medium or Low pile.
		 */
		public DrawCanvas(int type, int pile) {
			super();
			this.type = type;
			this.pile = pile;
			background = Toolkit.getDefaultToolkit().getImage("images\\pile.gif");
		    MediaTracker mediaTracker = new MediaTracker(this);
			mediaTracker.addImage(background, 0);
			try	{
				mediaTracker.waitForID(0);
				if(mediaTracker.isErrorID(0)) {
				    throw new InterruptedException("Unable to set pile background image");
				}
			} 
			catch (InterruptedException ie) {
			    JOptionPane.showMessageDialog(null, "Unable to set pile background image",
						"Failed", JOptionPane.ERROR_MESSAGE);
				ie.printStackTrace();
			}
		}
		
		/**
		 * Draws the selection square. Cards within this square will 
		 * be selected to be moved into a pile.
		 * 
		 * @param g the Graphics object to draw on
		 */
		private void drawSelectionLines(Graphics g) {
		    
		    if (startSelectionPoint != null && endSelectionPoint != null) {
		        g.setColor(new Color(125, 125, 255));
		        
		        int startX = startSelectionPoint.x;
		        int startY = startSelectionPoint.y;
		        int endX = endSelectionPoint.x;
		        int endY = endSelectionPoint.y;
		        
		        /* Draw a square by drawing four lines */
		        g.drawLine(startX, startY, endX, startY);
		        g.drawLine(startX, startY, startX, endY);        
		        g.drawLine(endX, startY , endX, endY);		        
		        g.drawLine(startX, endY , endX, endY);
		    }
		    
		}
		
		
		
		/**
		 * Paints all graphics 
		 */
		public void paint(Graphics g) {
		    /* Checks if the desk or a pile should be painted */
		    if(type == DESK) {
		        g.clearRect(0, 0, deskWidth, deskHeight);
		        g.drawImage(background, 0, 0, deskWidth, deskHeight, null);
		        pg.drawDesk(g, this);		        
		        drawSelectionLines(g);
		    }			
		    else {
		        
		        g.clearRect(0, 0, this.getWidth(), this.getHeight());
		        g.drawImage(background, 0, 0, this.getWidth(), this.getHeight(), null);
		        pg.drawPile(g, pile);
		    }
		}
	}
	
	/**
	 * MouseListener to handle user interaction
	 */
	class MouseHandler implements MouseMotionListener, MouseListener {
		public void mouseClicked(MouseEvent e) {
			if(e.getClickCount()==2 && e.getSource() == desk) {
			    /* Check if double-click on a pile */
				String selectedPile = (String) pg.selectPile(e.getX(), e.getY())[0];
				if (selectedPile.compareTo("High")==0) {
					highPile.setVisible(true);
					highPile.repaint();
				} 
				else if (selectedPile.compareTo("Medium")==0) {
					mediumPile.setVisible(true);
					mediumPile.repaint();
				} 
				else if (selectedPile.compareTo("Low")==0) {
					lowPile.setVisible(true);
					lowPile.repaint();
				}
			}
		}
		
		public void mouseMoved(MouseEvent e) {
		    Requirement r = pg.getRequirementAt(e.getPoint(), ((DrawCanvas) e.getSource()).pile);
		    if(r!=null) {
		        if (((DrawCanvas) e.getSource()).pile == PlanningGame.DESK) {
		            desk.setToolTipText(r.getReqNbr()+": "+r.getReq()); 
		        }
		        else if (((DrawCanvas) e.getSource()).pile == PlanningGame.HIGH){
		            highPile.pileCanvas.setToolTipText(r.getReqNbr()+": "+r.getReq()); 
		        }
		        else if (((DrawCanvas) e.getSource()).pile == PlanningGame.MEDIUM){
		            mediumPile.pileCanvas.setToolTipText(r.getReqNbr()+": "+r.getReq()); 
		        }
		        else if (((DrawCanvas) e.getSource()).pile == PlanningGame.LOW){
		            lowPile.pileCanvas.setToolTipText(r.getReqNbr()+": "+r.getReq()); 
		        }
		    }
		    else {
		        desk.setToolTipText(null);
		    }
		}
		
		public void mouseDragged(MouseEvent e) {
			if(e.getSource() == desk) {			
				if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) > 0) {
				    /* Drag a selected group of Cards */
				    if (desk.startSelectionPoint != null) {
			            desk.endSelectionPoint = e.getPoint(); 
			            pg.selectCardsInSelectionSquare(
			                    desk.startSelectionPoint, desk.endSelectionPoint);
			            desk.repaint();
			        }
					pg.moveSelectedCardsTo(e.getX(), e.getY(), desk.getSize());
				}
				desk.repaint();
			}
			/*
			 * If mouseDragged is done whithin a pile
			 */
			else if(e.getSource() == highPile.pileCanvas) {
				if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) > 0) {
					pg.moveSelectedInPileTo(e.getY(), PlanningGame.HIGH);
				}
				highPile.pileCanvas.repaint();
			} 
			else if(e.getSource() == mediumPile.pileCanvas) {
				if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) > 0) {
					pg.moveSelectedInPileTo(e.getY(), PlanningGame.MEDIUM);
				}
				mediumPile.pileCanvas.repaint();
			} 
			else if(e.getSource() == lowPile.pileCanvas) {
				if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) > 0) {
					pg.moveSelectedInPileTo(e.getY(), PlanningGame.LOW);
				}
				lowPile.pileCanvas.repaint();
			}
		}
 
		public void mouseEntered(MouseEvent e) { }

		public void mouseExited(MouseEvent e) {	}

		
		public void mousePressed(MouseEvent e) {
			if(e.getSource() == desk) {
			    desk.startSelectionPoint = null;
			    desk.endSelectionPoint = null;
			    
			    /* Check if the user wants to add an additional card to the selection */
			    if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) > 0 && e.isControlDown()) {
			        pg.addCardToSelectionList(e.getPoint());
			        Requirement r = pg.select(e.getX(), e.getY(), PlanningGame.DESK);
			        if (r==null) {
			            /* If the user wants to select a group of requirements */
					    desk.startSelectionPoint = e.getPoint();
			        }
			        setInfoAreaText(r);	
			    }
			    /* Check if click is on a card on the desk */
				else if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) > 0) {
				    Requirement r = pg.select(e.getX(), e.getY(), PlanningGame.DESK);
			        if (r==null) {
			            /* If the user wants to select a group of requirements */
					    desk.startSelectionPoint = e.getPoint();
			        }
			        setInfoAreaText(r);
				}
			}
			else if(e.getSource() == highPile.pileCanvas) {
			    /* Check if click is on a card in the High-pile */
				if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) > 0) {
					pg.deselectCard();
					setInfoAreaText(pg.select(e.getX(), e.getY(), PlanningGame.HIGH));
				}
				
			}
			else if(e.getSource() == mediumPile.pileCanvas) {
			    /* Check if click is on a card in the Medium-pile */
				if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) > 0) {
					pg.deselectCard();
					setInfoAreaText(pg.select(e.getX(), e.getY(), PlanningGame.MEDIUM));
				}
				
			}
			else if(e.getSource() == lowPile.pileCanvas) {
			    /* Check if click is on a card in the Low-pile */
				if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) > 0) {
					pg.deselectCard();
					setInfoAreaText(pg.select(e.getX(), e.getY(), PlanningGame.LOW));					
				}
				
			}
		    desk.repaint();
		    highPile.pileCanvas.repaint();
		    mediumPile.pileCanvas.repaint();
		    lowPile.pileCanvas.repaint();
		}
		
		
		public void mouseReleased(MouseEvent e) {
			if(e.getSource() == desk) {
			    desk.startSelectionPoint = null;
			    desk.endSelectionPoint = null;
				boolean cardDropped = pg.putCardInPile();
				/* Check if card has been dropped in a pile */
				if (cardDropped) {
					highPile.resizePileCanvas();
					mediumPile.resizePileCanvas();
					lowPile.resizePileCanvas();
				}
				/* Disable criteriaSelecter after first drop */
				if(criteriaSelecter.isEnabled() && cardDropped) {
					criteriaSelecter.setEnabled(false);
				}
				pg.unhighlightAllPiles();
				/* Check if all cards has been dropped in a pile */
				if(pg.getNbrOfCardsOnDesk()==0) {
					if (!prioOneDone) {
						nextCritButton.setEnabled(true);
						nextCriteriaMenuItem.setEnabled(true);
					} else {
						doneButton.setEnabled(true);
						doneMenuItem.setEnabled(true);
					}
				}
			}
			else if(e.getSource() == highPile.pileCanvas) {
				pg.moveCardToEmptyPlace();
				highPile.pileCanvas.repaint();
			} 
			else if(e.getSource() == mediumPile.pileCanvas) {
				pg.moveCardToEmptyPlace();
				mediumPile.pileCanvas.repaint();
			} 
			else if(e.getSource() == lowPile.pileCanvas) {
				pg.moveCardToEmptyPlace();
				lowPile.pileCanvas.repaint();
			}
		}
	}
	
	/**
	 * Window for sorting the cards internally in a pile
	 */
	class PileWindow extends JFrame {
		private LinkedList cardsList;
		private DrawCanvas pileCanvas;
		private Container pileContentPane;
		private JLabel pileHeader;
		
		private String pileName;
		private JPanel pileNorth, pileCenter, pileSouth;
		private JButton pileOkButton, pileToDeskButton;
		private JScrollPane pileScrollPane;
		
		/**
		 * Constructs a PileWindow
		 * 
		 * @param pileName the name of the pile: "High", "Medium" or "Low"
		 * @param cardsList the list of cards in the current pile
		 */
		public PileWindow(String pileName, LinkedList cardsList) {
			super(pileName);
			this.pileName = pileName;
			this.cardsList = cardsList;
			
			if (pileName.compareTo("High")==0) {
				initPileWindow(PlanningGame.HIGH);
			} 
			else if (pileName.compareTo("Medium")==0) {
				initPileWindow(PlanningGame.MEDIUM);
			} 
			else if (pileName.compareTo("Low")==0) {
				initPileWindow(PlanningGame.LOW);
			}
		}
		
		/**
		 * Initializes the graphical user interface
		 * 
		 * @param pileType the type of the pile used when constructing
		 * the corresponding drawCanvas
		 */
		private void initPileWindow(int pileType) {
			this.setSize(350, deskHeight);
			this.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
			this.setResizable(false);
			pileContentPane = this.getContentPane();
			pileContentPane.setLayout(new BorderLayout());
			this.addWindowListener(wh);
			setIconImage(getToolkit().getImage("images\\rainbow.gif"));

			
			/* North panel */
			pileHeader = new JLabel(pileName);
			Util.setFont(pileHeader, true, Util.FONT_MEDIUM);
			pileNorth = new JPanel();
			pileNorth.add(pileHeader);
			pileContentPane.add(pileNorth, BorderLayout.NORTH);
			
			/* Center panel */
			pileCenter = new JPanel();
			pileCenter.setLayout(new BoxLayout(pileCenter, BoxLayout.X_AXIS));
			pileCanvas = new DrawCanvas(DrawCanvas.PILE, pileType);
			pileCanvas.addMouseListener(mh);
			pileCanvas.addMouseMotionListener(mh);
			
			pileScrollPane = new JScrollPane(pileCanvas, 
			        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
			        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);	
			
			pileCenter.add(Box.createHorizontalGlue());
			pileCenter.add(pileScrollPane);
			pileCenter.add(Box.createHorizontalGlue());
			pileContentPane.add(pileCenter, BorderLayout.CENTER);
			
			/* South panel */
			pileSouth = new JPanel();
			pileSouth.setLayout(new BoxLayout(pileSouth, BoxLayout.X_AXIS));
			pileSouth.setBorder(BorderFactory.createEmptyBorder(10, 0, 20, 0));
			pileOkButton = new JButton("Ok");
			pileToDeskButton = new JButton("Send to desk");
			Util.setJComponentSize(pileOkButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
			Util.setJComponentSize(pileToDeskButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
			pileOkButton.addActionListener(ah);
			pileToDeskButton.addActionListener(ah);
			pileSouth.add(Box.createHorizontalGlue());
			pileSouth.add(pileToDeskButton);
			pileSouth.add(Box.createHorizontalStrut(20));
			pileSouth.add(pileOkButton);
			pileContentPane.add(pileSouth, BorderLayout.SOUTH);
			pileSouth.add(Box.createHorizontalGlue());
			
			this.setVisible(true);
			
			resizePileCanvas();
			/* 
			 * Window is created immediately but not visible until the 
			 * user opens it the first time
			 */
			this.setVisible(false);
		}
		
		/**
		 * Resizing the drawCanvas every time a new card is added or removed
		 */
		private void resizePileCanvas() {
			pileCanvasHeight = cardsList.size()*(PlanningGame.CARD_HEIGHT+20) + 25;

			Util.setJComponentSize(pileCanvas, 270, pileCanvasHeight);
			Util.setJComponentSize(pileScrollPane, 270, pileCenter.getHeight());
		    
		    pileScrollPane.setViewportView(pileCanvas);

		    pileCanvas.repaint();
		}
		
		
		/**
		 * Sets the list of cards in the current pile window
		 * 
		 * @param cardsList the list of cards
		 */
		public void setCardsList(LinkedList cardsList) {
			this.cardsList = cardsList;
		}
	}
	
	/**
	 * Window used to get a confirmation from the user that all requirements are
	 * listed in the correct order. Main purpose of this window is to avoid
	 * "head-tail" problems.
	 */
	class ConfirmWindow extends JFrame {
	    
	    private JButton acceptPrioButton, backPrioButton, assignValuesButton;
	    
	    /**
	     * Constructs a ConfirmWindow
	     */
	    public ConfirmWindow() {
	        super("Confirm priorities");
	        init();
	    }
	    
	    /**
	     * Inititalizes the graphical user interface
	     */
	    private void init() {
	        
	        setSize(600, 400);
	        setResizable(false);
	        setDefaultCloseOperation(DISPOSE_ON_CLOSE);        
	        Container contentPane = getContentPane();
	        contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
	        setIconImage(getToolkit().getImage("images\\rainbow.gif"));
	        
	        /* Header */
	        JPanel headerPanel = new JPanel();
	        JLabel header = new JLabel("Confirm order for " + currentCriteria);
	        header.setFont(new Font("Dialog", Font.BOLD, 16));
	        headerPanel.add(header);	        
	        
	        /* List of requirments in prioritized order */
	        JPanel listPanel = new JPanel();
	        listPanel.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 10));
	        DefaultListModel listModel = new DefaultListModel();
	        listModel.addElement("High:");
	        for(int h=0;h<pg.getPile("High").size();h++) {
	            listModel.addElement(pg.getRequirement("High", h).getReqNbr()+
	                    ": "+pg.getRequirement("High", h).getReq());
	        }
	        listModel.addElement("Medium:");
	        for(int m=0;m<pg.getPile("Medium").size();m++) {
	            listModel.addElement(pg.getRequirement("Medium", m).getReqNbr()+
	                    ": "+pg.getRequirement("Medium", m).getReq());
	        }
	        listModel.addElement("Low:");
	        for(int l=0;l<pg.getPile("Low").size();l++) {
	            listModel.addElement(pg.getRequirement("Low", l).getReqNbr()+
	                    ": "+pg.getRequirement("Low", l).getReq());
	        }
	        JList reqList = new JList(listModel);
	        reqList.setEnabled(false);
	        JScrollPane scrollPane = new JScrollPane(reqList);
	        scrollPane.setPreferredSize(new Dimension(600, 400));
	        listPanel.add(scrollPane);
	        
	        /* Buttons */
	        JPanel buttonsPanel = new JPanel();
	        buttonsPanel.setBorder(BorderFactory.createEmptyBorder(5,15,15,15));
	        assignValuesButton = new JButton("Assign values");
	        Util.setJComponentSize(assignValuesButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
	        assignValuesButton.addActionListener(ah);
	        assignValuesButton.setToolTipText("Assign relative values using $100 technique");
	        acceptPrioButton = new JButton("Confirm");
	        Util.setJComponentSize(acceptPrioButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
	        acceptPrioButton.addActionListener(ah);
	        acceptPrioButton.setToolTipText("Accept the prioritization order");
	        backPrioButton = new JButton("Back");
	        Util.setJComponentSize(backPrioButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
	        backPrioButton.addActionListener(ah);
	        backPrioButton.setToolTipText("Return to the desk to change the prioritization " +
	        		"order");
	        buttonsPanel.setLayout(new BoxLayout(buttonsPanel, BoxLayout.X_AXIS));
	        
	        buttonsPanel.add(assignValuesButton);
	        buttonsPanel.add(Box.createHorizontalGlue());
	        buttonsPanel.add(acceptPrioButton);
	        buttonsPanel.add(Box.createHorizontalStrut(20));
	        buttonsPanel.add(backPrioButton);
	        
	        contentPane.add(headerPanel);
	        contentPane.add(listPanel);
	        contentPane.add(buttonsPanel);
	        
	        /* 
	         * If the user earlier has made the choice to use "Assign values", 
	         * both criteria must use this method
	         */
	        if(pg.getRequirementList().getPrioMethod()==RequirementList.DOLLAR) {
	            acceptPrioButton.setEnabled(false);
	        }
	        
	        /* 
	         * If the user earlier has made the choice to not use "Assign values", 
	         * neither criteria can use this method
	         */
	        if(pg.getRequirementList().getPrioMethod()==RequirementList.PLANNING_GAME &&
	                prioOneDone) {
	            assignValuesButton.setEnabled(false);
	        }
	        
	        pack();
	        Util.centerJFrame(this);
	        setVisible(true);
	    }   	    
	}
	
	/**
	 * WindowHandler that takes care of Window Events for
	 * PlanningGameWindow and PileWindow.
	 */
	class WindowHandler implements WindowListener {
		public void windowActivated(WindowEvent e) {
		    /*
		     * When a pile window is activated it must be resized
		     * (due to possible card transfers) and all existing 
		     * pile windows shall be repainted.
		     */
			if (e.getSource() instanceof PileWindow) {
				((PileWindow) e.getSource()).resizePileCanvas();
				if ( highPile != null) {
					highPile.pileCanvas.repaint();
				}
				if (mediumPile != null) {
					mediumPile.pileCanvas.repaint();
				}
				if (lowPile != null) {
					lowPile.pileCanvas.repaint();
				}
			} 
		
		}
		public void windowClosed(WindowEvent e) { }
		public void windowClosing(WindowEvent e) {
		    /* 
		     * When the main planning game window is closed, all pile windows
		     * must be disposed 
		     */
			if (e.getSource() instanceof PlanningGameWindow) {
				highPile.dispose();
				mediumPile.dispose();
				lowPile.dispose();
			}
		}
		public void windowDeactivated(WindowEvent e) { }
		public void windowDeiconified(WindowEvent e) { }
		public void windowIconified(WindowEvent e) { }
		public void windowOpened(WindowEvent e){ }
	}
}
