/*
 * ExcelImporter.java
 *
 * Created on Jan 21, 2005
 */

package importer;

import gui.ExcelImporterWindow;
import gui.ImportWindow;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JOptionPane;

import rainbowie.*;

import jxl.*;
import jxl.read.biff.BiffException;

/**
 * Defines an importer used for importing requirements from MS Excel
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class ExcelImporter {

	private Workbook workbook;
	private ExcelImporterWindow eiw;
	private Sheet sheet;
	private ImportWindow iw;

	/**
	 * Constructs an ExcelImporter
	 * 
	 * @param iw		the window used for user interaction 
	 * during import process
	 * @param excelFile the Excel file to import from
	 * @param p reference to the programs Parseq object
	 */
	public ExcelImporter(ImportWindow iw, File excelFile, Rainbowie p) {
		try {
			workbook = Workbook.getWorkbook(excelFile);
		} 
		catch (IOException e) { 
			JOptionPane.showMessageDialog(null, "Unable to open Excel file",
					"Failed", JOptionPane.ERROR_MESSAGE);
		}
		catch (BiffException be) {
		    JOptionPane.showMessageDialog(null, "Unable to open Excel file",
					"Failed", JOptionPane.ERROR_MESSAGE);
		}
		sheet = workbook.getSheet(0);
		eiw = new ExcelImporterWindow(this, sheet.getRows(), sheet.getColumns(), p);
		this.iw = iw;		
		setupExcelImporterWindow();
	}

	/**
	 * Sets the Excel spreadsheet to use for import
	 * @param n the index of the spreadsheet to use
	 */
	public void setSheet(int n) {
	    sheet = workbook.getSheet(n);
	}
	
	/**
	 * Returns the names of the spreadsheets in the workbook
	 * 
	 * @return an array containg the names of the spreadsheets
	 */
	public String[] getSheets() {
	    return workbook.getSheetNames();
	}
	
	/**
	 * Returns the number of rows in the current spreadsheet
	 * 
	 * @return the spreadsheets number of rows
	 */
	public int getRows() {
	    return sheet.getRows();
	}
	
	/**
	 * Returns the number of columns in the current spreadsheet
	 * 
	 * @return the spreadsheets number of columns
	 */
	public int getCols() {
	    return sheet.getColumns();
	}
	
	/**
	 * Setup the interaction window with the correct data and number of
	 * rows and columns.
	 */
	public void setupExcelImporterWindow() {
	    for (int r = 0; r < sheet.getRows(); r++) {
			for (int c = 0; c < sheet.getColumns(); c++) {
				Cell cell = sheet.getCell(c, r);
				String s = cell.getContents();
				eiw.setCellAt(s, r, c);
			}
	    }
	    eiw.setVisible(true);
	}
	
	/**
	 * Imports the requirements defined by the user
	 * 
	 * @param vector the data vector of the table
	 * @param indexes the index array containg information of the rows
	 * and columns where the requirements to import can be found
	 */
	public void importRequirements(Vector vector, int[] indexes) {
	    RequirementList rList = new RequirementList();
	    for(int i=0;i<(indexes[1]-indexes[0]+1); i++) {
	        Requirement req = new Requirement();
	        
	        /* Add requirement nbr. info */
	        req.setReqNbr((String) ((Vector) vector.get(indexes[0]+i)).get(indexes[2]));
	        
	        /* Add requirement info */
	        if(indexes[3]!=-1) {
	            req.setReq((String) ((Vector) vector.get(indexes[0]+i)).
	                    get(indexes[3]));
	        }
	        else {
	            req.setReq("");
	        }
	        
	        /* Add release nbr. info */
	        if(indexes[4]!=-1) {
	            req.setRelNbr((String) ((Vector) vector.get(indexes[0]+i)).
	                    get(indexes[4]));
	        }
	        else {
	            req.setRelNbr("");
	        }
			
			rList.addRequirement(req);
	    }
	    iw.updateReqTable(rList);
	}
}