/*
 * Parseq.java
 *
 * Created on Jan 21, 2005
 */

package rainbowie;

import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import util.HelpDialog;

import gui.*;
import gui.prio.*;

/**
 * Defines the Parseq object of the program. The main object.<br>
 * This is where the lists of imported and prioritized requirements 
 * are held.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class Rainbowie {
	
	private RequirementList importedList, prioList;
	private MainWindow mw;
	private JFrame lastPrioWindow;
	private HelpDialog hd;
	
	/**
	 * Constructs the Parseq object and instantiates
	 * a HelpDialog object and the MainWindow
	 */
	public Rainbowie() {
	    hd = new HelpDialog();
		mw = new MainWindow(this);		
	}
	
	/**
	 * Returns the list of imported requirements
	 * 
	 * @return the imported requirement list.
	 */
	public RequirementList getImportedList() {
		return importedList;
	}
	
	/**
	 * Returns the list of prioritized requirements.
	 * 
	 * @return the prioritized requirements list.
	 */
	public RequirementList getPrioList() {
		return prioList;
	}
	
	public void setLastPrioWindow(JFrame lastPrioWindow) {
	    this.lastPrioWindow = lastPrioWindow;
	}
	
	public JFrame getLastPrioWindow() {
	    return lastPrioWindow;
	}
	
	/**
	 * Sets the list of imported requirements
	 * 
	 * @param v the Vector object containing the list to set.
	 */
	public void setImportedList(Vector v) {
		RequirementList list = new RequirementList();
		for(int i=0;i<v.size();i++) {
			Requirement r = new Requirement();
			r.setReqNbr(((Vector) v.get(i)).get(0).toString());
			r.setReq(((Vector) v.get(i)).get(1).toString());
			r.setRelNbr(((Vector) v.get(i)).get(2).toString());
			list.addRequirement(r);
		}
		importedList = list;
		mw.setImportedTable(importedList);
	}
	
	/**
	 * Sets the list of prioritized requirements
	 * 
	 * @param prioList the RequirementList object containing the list to set.
	 */
	public void setPrioList(RequirementList prioList) {
		this.prioList = prioList;
		mw.setPrioTable(prioList);
	}
	
	/**
	 * Sets the prioritization method and criteria.<br>
	 * The method is called when the user has chosen method and 
	 * criteria before a new prioritization
	 * 
	 * @param prioMethod  	the prio method to use
	 * @param prioCriteria1 the first criteria to use
	 * @param prioCriteria2 the second criteria to use
	 */
	public void setPrioParams(int prioMethod, String prioCriteria1, String prioCriteria2) {   
		if (importedList != null) {
		    RequirementList tempList = importedList.getClampedCopy();
			tempList.setPrioMethod(prioMethod);
			tempList.setPrioCriteria1(prioCriteria1);
			tempList.setPrioCriteria2(prioCriteria2);
			new PrioWindowStarter(tempList, this);
		} 
		else {
			JOptionPane.showMessageDialog(null, "There seem to be no requirments to" +
					" prioritize.\n\r Please go back and import or load requirments" +
					" before continuing.", "Failed - No Requirements",
					JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/**
	 * Shows the help dialog with the requested page to use as start page
	 * 
	 * @param helpfile the page to show when the dialog is visible
	 */
	public void showHelpDialog(String helpfile) {
	    hd.showHelpText(helpfile);
	}
	
	
	/**
	 * The main method of the program. Instantiates a Parseq object.
	 * 
	 * @param args the arguments given. Not used.
	 */
	public static void main(String[] args) {
	    /* Sets the Look n' Feel to system default */
	    try {
	        UIManager.setLookAndFeel(
	            UIManager.getSystemLookAndFeelClassName());
	    } 
	    catch (Exception e) {
	        JOptionPane.showMessageDialog(null, "Unable to set Look n' Feel",
					"Failed", JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
	    }
		new Rainbowie();		
	}
}
