/*
 * Filter.java
 *
 * Created on 2005-feb-22
 */

package util;

import java.io.File;
import javax.swing.filechooser.FileFilter;

/**
 * Defines a file filter that can be used when opening and saving files. The 
 * filter is used to specify what files to show in the file dialog.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class Filter extends FileFilter {
	private String filter;
	
	/**
	 * Constructs the Filter.
	 * 
	 * @param filter a string containing the file ending you want to filter
	 */
	public Filter(String filter) {
		this.filter = filter;
	}
	
	/**
	 * Checks if the specified file should be shown or not.
	 * 
	 * @param f the file to be checked
	 * @return true if the file should be shown; else false
	 */
	public boolean accept(File f) {
	    /* Always show directories */
		if (f.isDirectory()) {
			return true;
		}
		String extension = getExtension(f);
		if (extension != null) {
			if (extension.equals(filter)) {
				return true;
			} 
			return false;
		}
		return false;
	}
	
	/**
	 * The filter's description.
	 * 
	 * @return the filter's description
	 */
	public String getDescription() {
		return "Just *." + filter;
	}
	
	/**
     * Get the extension (file ending) of a file.
     * 
     * @param f the file which filename we want
     * @return the file's extension
     */  
    public String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }
}