/*
 * HelpDialog.java
 *
 * Created on 2005-mar-01
 */

package util;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;

/**
 * Defines the class used to display the help dialogs.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class HelpDialog {

    private HelpWindow helpWindow;
    private JEditorPane infoPane;
    private JList helpTopics;
    private String[] helpFiles;
    
    /**
     * Constructs a help dialog.
     */
    public HelpDialog() {
        helpWindow = new HelpWindow();
    }

    /**
     * Gets the available help topics.
     * 
     * @return an array with the available topics
     */
    private String[] getHelpTopics() {
        /* The data displpayed in the list on the left hand side */
        String[] data = {
                "Main Window", 
                "Import Window", 
                " - Excel Import Window",
                "Priority Method Chooser",
                "Planning Game",
                "Pairwise Comparisons",
                "$100 Technique",
                "Root-Cause Analysis",
                " - Graph Window",
                " - Root-Cause Matrix"};
        
        /* 
         * Used for linking the help files displayed on the right hand side
         * to the data list on the left hand side 
         */
        helpFiles = new String[data.length];
        helpFiles[0] = "mainwindow.html";
        helpFiles[1] = "importwindow.html";
        helpFiles[2] = "excelimporterwindow.html";
        helpFiles[3] = "choosepriowindow.html";
        helpFiles[4] = "planninggamewindow.html";
        helpFiles[5] = "pairwisewindow.html";
        helpFiles[6] = "dollarwindow.html";
        helpFiles[7] = "rootcauseanalysis.html";
        helpFiles[8] = "graphwindow.html";
        helpFiles[9] = "improvementswindow.html";
        return data;
    }
    
    /**
     * Show the selected help file in the HelpWindow.
     *  
     * @param helpfile the file to be shown
     */
    public void showHelpText(String helpfile) {
        try {
            java.net.URL helpURL = new java.net.URL("file:helpfiles//"+helpfile);
            infoPane.setPage(helpURL);
        }
        catch (Exception e) {
            infoPane.setText(helpfile+" not found");
        }
        for(int i=0;i<helpFiles.length;i++) {
            if(helpfile.compareTo(helpFiles[i])==0) {
                helpTopics.setSelectedIndex(i);
            }
        }
        helpWindow.setVisible(true);
    }
    
    /**
     * The class defining the window to show the help in.
     */
    class HelpWindow extends JFrame {
        
        private Container contentPane;
        private ListHandler lh;
        private WindowHandler wh;
        private ActionHandler ah;
        private JButton closeButton;
        private JScrollPane helpTopicsScrollPane;
        
        /**
         * Constructs the help window.
         */
        public HelpWindow() {
            super("Help");
            ah = new ActionHandler();
            lh = new ListHandler();
            wh = new WindowHandler();
            this.addWindowListener(wh);
            this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
            init();
        }

        /**
         * Initializes the graphical user interface.
         */
        private void init() {
            contentPane = getContentPane();
            contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
            setIconImage(getToolkit().getImage("images\\ques2.gif"));

            /* Center */
            JPanel center = new JPanel();
            center.setLayout(new BoxLayout(center, BoxLayout.X_AXIS));
            center.setAlignmentX(Box.RIGHT_ALIGNMENT);
            center.setBorder(BorderFactory.createEmptyBorder(0,10,10,10));
            
            /* Bottom */
            JPanel bottom = new JPanel();
            bottom.setLayout(new BoxLayout(bottom, BoxLayout.X_AXIS));
            closeButton = new JButton("Close");
            Util.setJComponentSize(closeButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
            closeButton.addActionListener(ah);
            bottom.setAlignmentX(Box.RIGHT_ALIGNMENT);
            bottom.add(closeButton);
            bottom.setBorder(BorderFactory.createEmptyBorder(5 ,10 , 10, 10));
            
            /* Left */
            JPanel left = new JPanel();
            left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
            left.setPreferredSize(new Dimension(175, 400));
            left.setMaximumSize(new Dimension(200, 4096));
            
            JLabel leftLabel = new JLabel("Choose topic:");
            leftLabel.setAlignmentX(Box.LEFT_ALIGNMENT);
            
            helpTopics = new JList(getHelpTopics());
            helpTopicsScrollPane = new JScrollPane(helpTopics);
            helpTopicsScrollPane.setPreferredSize(new Dimension(175, 400));
            helpTopicsScrollPane.setAlignmentX(Box.LEFT_ALIGNMENT);
            helpTopics.addListSelectionListener(lh);
            
            left.add(Box.createVerticalStrut(5));
            left.add(leftLabel);
            left.add(Box.createVerticalStrut(5));
            left.add(helpTopicsScrollPane);
            
            /* Right */
            JPanel right = new JPanel();
            right.setLayout(new BoxLayout(right, BoxLayout.Y_AXIS));
            JLabel rightLabel = new JLabel(" ");
            
            infoPane = new JEditorPane();
            infoPane.setEditable(false);
            infoPane.addHyperlinkListener(new Hyperactive());
            JScrollPane infoPaneScrollPane = new JScrollPane(infoPane);
            infoPaneScrollPane.setPreferredSize(new Dimension(325, 400));

            right.add(Box.createVerticalStrut(5));
            right.add(rightLabel);
            right.add(Box.createVerticalStrut(5));
            right.add(infoPaneScrollPane);
            
            center.add(left);
            center.add(Box.createHorizontalStrut(10));
            center.add(right);
            
            contentPane.add(center);
            contentPane.add(bottom);
            
            pack();
            setVisible(false);
        }
    }
    
    /**
     * Defines a ListSelectionLister that is used to detect on which topic 
     * the user clicks. 
     */
    class ListHandler implements ListSelectionListener {
        
        public void valueChanged(ListSelectionEvent e) {
            /* If the new topic is above the previous topic in the list */
            if(((JList) e.getSource()).getSelectedIndex() == e.getFirstIndex()) {
                showHelpText(helpFiles[e.getFirstIndex()]);
            }
            /* If the new topic is below the previous topic in the list */
            else if(((JList) e.getSource()).getSelectedIndex() == e.getLastIndex()) {
                showHelpText(helpFiles[e.getLastIndex()]);
            }
        }
    }
    
    /**
     * Used to hide the window when the user closes it by clicking the 
     * close icon in the titlebar. 
     */
    class WindowHandler implements WindowListener {
        
        public void windowActivated(WindowEvent e) {}
        public void windowClosed(WindowEvent e) {}
        public void windowClosing(WindowEvent e) {
            helpWindow.setVisible(false);
        }
        public void windowDeactivated(WindowEvent e) {}
        public void windowDeiconified(WindowEvent e) {}
        public void windowIconified(WindowEvent e) {}
        public void windowOpened(WindowEvent e) {}
    }
    
    /**
     * Used to hide the window when the user closes it by clicking the 
     * close button. 
     */
    class ActionHandler implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            if(e.getSource() == helpWindow.closeButton) {
                helpWindow.setVisible(false);
            }
        }
    }

    /**
     * Class used to handle the hyperlinks in the HelpWindow.
     */
    class Hyperactive implements HyperlinkListener {
        
        public void hyperlinkUpdate(HyperlinkEvent e) {
            if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                JEditorPane pane = (JEditorPane) e.getSource();
                if (e instanceof HTMLFrameHyperlinkEvent) {
                    HTMLFrameHyperlinkEvent  evt = (HTMLFrameHyperlinkEvent)e;
                    HTMLDocument doc = (HTMLDocument)pane.getDocument();
                    doc.processHTMLFrameHyperlinkEvent(evt);
                } 
                else {
                    String url = "";
                    try {
                        pane.setPage(e.getURL());
                        
                        url = e.getURL().getFile();
                        for(int i=0;i<helpFiles.length;i++) {
                            if(url.endsWith(helpFiles[i])) {
                                helpTopics.setSelectedIndex(i);
                            }
                        }
                    } 
                    catch (Throwable t) {
                        infoPane.setText(url+" not found");
                    }
                }
            }
        }
    }


}
