/*
 * Util.java
 *
 * Created on 2005-feb-14
 */

package util;

import java.awt.AWTError;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.util.StringTokenizer;

import javax.swing.JComponent;
import javax.swing.JFrame;

/**
 * Defines a class containing commonly used utility methods.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class Util {
    /* Button dimensions to be used throughout the program */
    public final static int BUTTON_WIDTH = 125, BUTTON_HEIGHT= 30;
    /* Font sizes to be used throughout the program */
    public final static int FONT_XSMALL = 10, FONT_SMALL = 14, 
    FONT_MEDIUM = 16, FONT_LARGE = 24;
    /* Button distances to be used throughout the program */
    public final static int HORIZONTAL_BUTTON_DISTANCE = 20, VERTICAL_BUTTON_DISTANCE = 15;
    
    
    /**
     * Forces the size of a JComponent.
     * 
     * @param c      the component to set the size on
     * @param width  the width to set
     * @param height the height to set 
     */
    public static void setJComponentSize(JComponent c, int width, int height) {
		c.setPreferredSize(new Dimension(width, height));
		c.setMinimumSize(new Dimension(width, height));
		c.setMaximumSize(new Dimension(width, height));
	}
    
    /**
     * Sets the font for the specific JComponent
     * 
     * @param comp the current JComponent
     * @param bold true if font should be set to bould, plain if false
     * @param size the size of the font
     */
    public static void setFont(JComponent comp, boolean bold, int size) {
        int style = (bold ? Font.BOLD : Font.PLAIN);
        comp.setFont(new Font("Dialog", style, size));
    }
    
    /**
     * Makes sure that the frame is opened in the middle of the screen.
     * 
     * @param jFrame the frame to center
     */
    public static void centerJFrame(JFrame jFrame) {
        Dimension frameSize = jFrame.getSize();
        Dimension screenSize;
        try {
            Toolkit tk = Toolkit.getDefaultToolkit();
            screenSize = tk.getScreenSize();
        }
        catch (AWTError awe) {
            screenSize = new Dimension (640, 480);
        }
                                                       
        if (frameSize.height > screenSize.height) {
           frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width) {
           frameSize.width = screenSize.width;
        }
                                                                                  
        /* Center the JFrame */
        int x = screenSize.width/2 - frameSize.width/2;
        int y = screenSize.height/2 - frameSize.height/2;
        jFrame.setLocation(x,y);
    }
	
	/**
	 * Cuts off a String to the desired length ending it with "...".
	 * If rows > 1 the method will return a String where each row has 
	 * the desired length.
	 * 
	 * @param str       the String to be shortened
	 * @param length    the desired length
	 * @param nbrOfRows number of rows (line breaks) in the returned string
	 * @return          the shortened String
	 */
	public static String shortString(String str, int length, int nbrOfRows) {
		String shortString = "";
		String row = "";
		String token = "";
		boolean thisRow = true;
		if (str.length()>length) {
			StringTokenizer st = new StringTokenizer(str);
			for (int i=0;i<nbrOfRows;i++) {
				thisRow = true;
				while (st.hasMoreTokens() && thisRow) {
					token = st.nextToken();
					if(row.length() + token.length() < length ) {
						row = row + token + " ";
					}
					else if(i == nbrOfRows-1) {
						shortString = shortString + (row + token).substring(0,length-4) + "...";
						row = "";
						thisRow = false;
					}
					else {
						if(token.length()>length && row.length()>0) {
							shortString = shortString + row + "\n";
							thisRow = false;
							row = "";
							shortString = shortString + token.substring(0, length-1) + "-\n";
							row = token.substring(length, token.length()-1);
						}
						else if(token.length()>length) {
							shortString = shortString + token.substring(0, length-1) + "-\n";
							thisRow = false;
							row = "";
							row = token.substring(length, token.length()-1);
						}
						else {
							shortString = shortString + row + "\n";
							thisRow = false;
							row = "";
							row = row + token + " ";
						}
					}
					if(!st.hasMoreTokens()) {
						shortString = shortString + row;
					}
				}				
			}
		}
		else {
			return str;
		}
		return shortString;
	}
}
